#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#       
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#       
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from . import helpers
from . import usecases
import requests, logging

logger = logging.getLogger(__name__)

class AtlassianInterface(object):
    def __init__(self, credentials, location):
        self._credentials = credentials
        self._base_url = location.get_url()
        self._session = requests.session()

    def _perform_request(self, request_type, url_path, params = {}, json_data = {}):
        url = self._base_url + url_path
        headers={'Accept': 'application/json'}
        auth = (self._credentials.get_username(),
                self._credentials.get_password())
        result = None
    
        if request_type == 'GET':
            logger.info('GET %s with parameters %s', url, params)
        
            result = self._session.get(url, params = params, headers = headers, auth = auth)
        elif request_type == 'POST':
            logger.info('POST %s with parameters %s and json %s', url, params, json_data)
        
            result = self._session.post(url, params = params, headers = headers, auth = auth, json = json_data)
        elif request_type == 'PUT':
            logger.info('PUT %s with parameters %s and json %s', url, params, json_data)
        
            result = self._session.put(url, params = params, headers = headers, auth = auth, json = json_data)
        else:
            raise helpers.IllegalRequestTypeException('The specified request type is unknown: %s' % request_type)
            
        if (logger.isEnabledFor(logging.DEBUG)):
            logger.debug('Returned result: %s', result)
            logger.debug('Returned headers: %s', result.headers)
            
        if (result.status_code == 401):
            raise helpers.UnauthorizedException('Unauthorized: the request returned a status code 401: %s' % (result))
        if (result.status_code == 403):
            raise helpers.ForbiddenException('Forbidden: the request returned a status code 403: %s' % (result))
        elif (result.status_code == 404):
            raise helpers.ObjectNotFoundException('Not found: the request returned a status code 404: %s' % (result))
        else:
            if request_type == 'GET':
                if result.status_code == 200:
                    pass
                else:
                    raise helpers.UnknownResponseException('The request returned an unknown status code %s: %s' % (result.status_code, result))
            elif request_type == 'POST':
                if result.status_code == 200 or result.status_code == 201 or result.status_code == 204:
                    pass
                elif result.status_code == 400 or result.status_code == 409:
                    raise helpers.ObjectExistanceException('The object you want to reference has a wrong existance status: %s' % result)
                elif (result.status_code == 500):
                    raise helpers.ObjectNotFoundException('Not found: the request returned a status code 500: %s' % (result))
                else:
                    raise helpers.UnknownResponseException('The request returned an unknown status code %s: %s' % (result.status_code, result))
            elif request_type == 'PUT':
                if result.status_code == 204 or result.status_code == 200:
                    pass
                elif result.status_code == 400:
                    raise helpers.FailedUpdateException('The update failed: %s' % result)
                elif (result.status_code == 500):
                    raise helpers.ObjectNotFoundException('Not found: the request returned a status code 500: %s' % (result))
                else:
                    raise helpers.UnknownResponseException('The request returned an unknown status code %s: %s' % (result.status_code, result))
        return result
    
    def _perform_get(self, url_path, params = {}):
        return self._perform_request('GET', url_path, params=params)
    
    def _perform_post(self, url_path, params = {}, json_data={}):
        return self._perform_request('POST', url_path, params = params, json_data = json_data)
    
    def _perform_put(self, url_path, params = {}, json_data={}):
        return self._perform_request('PUT', url_path, params = params, json_data = json_data)

class CrowdInterface(AtlassianInterface):
    def __init__(self, credentials, location):
        AtlassianInterface.__init__(self, credentials, location)
        
    def get_user(self, user_id):
        result = self._perform_get('/rest/usermanagement/1/user', {'username': str(user_id), 'expand': 'attributes'})
        
        found_user = helpers.UserObject()
        found_user.load_json(result.json())
        
        return found_user
        
    def get_group(self, group_id, expand_attributes=True):
        parameters = {'groupname': str(group_id)}
        
        if expand_attributes:
            parameters['expand'] = 'attributes'
        
        result = self._perform_get('/rest/usermanagement/1/group', parameters)
        
        found_group = helpers.GroupObject()
        found_group.load_json(result.json())
        
        return found_group

    def get_child_groups_of_group(self, group_id):
        result = self._perform_get('/rest/usermanagement/1/group/child-group/direct', {'groupname': str(group_id)})
        
        found_group_list = helpers.GroupListObject()
        found_group_list.load_json(result.json())
        
        return found_group_list

    def get_parent_groups_of_group(self, group_id):
        result = self._perform_get('/rest/usermanagement/1/group/parent-group/direct', {'groupname': str(group_id)})
        
        found_group_list = helpers.GroupListObject()
        found_group_list.load_json(result.json())
        
        return found_group_list

    def get_nested_members_of_group(self, group_id):
        result = self._perform_get('/rest/usermanagement/1/group/user/nested', {'groupname': str(group_id)})
        
        found_user_list = helpers.UserListObject()
        found_user_list.load_json(result.json())
        
        return found_user_list

    def add_group(self, group_id, group_description):
        result = self._perform_post('/rest/usermanagement/1/group', {}, {'name': str(group_id), 'description': str(group_description), 'type': 'GROUP', 'active': True})
        
        found_group = helpers.GroupObject()
        found_group.load_json(result.json())
        
        return found_group
        
    def add_direct_child_group_membership(self, parent_group_id, child_group_id):
        self._perform_post('/rest/usermanagement/1/group/child-group/direct', {'groupname': str(parent_group_id)}, {'name': str(child_group_id)})
    
        return True
        
    def add_user(self, user_id, email, first_name, last_name, display_name, password):
        result = self._perform_post('/rest/usermanagement/1/user', {'expand': 'attributes'}, 
            {'name': user_id, 'password': {'value': password}, 'active': True, 'first-name': first_name,
                'last-name': last_name, 'display-name': display_name, 'email': email})
        
        found_user = helpers.UserObject()
        found_user.load_json(result.json())
        
        return found_user
        
    def store_user_attributes(self, user_id, attributes_dict):
        if type(attributes_dict).__name__ != 'dict':
            raise helpers.IllegalArgumentsException('Attributes to be stored should be in a dict!')
        elif len(attributes_dict) == 0:
            raise helpers.IllegalArgumentsException('At least one attribute to be stored should be specified!')
        
        parameters = {'username': str(user_id)}
        attributes = [{'name': str(attribute), 'values': [str(attributes_dict[attribute])]} for attribute in attributes_dict]
        
        self._perform_post('/rest/usermanagement/1/user/attribute', parameters, {'attributes': attributes})
        
        return True
        
    def add_user_to_group(self, user_id, group_id):
        parameters = {'username': str(user_id)}
        
        self._perform_post('/rest/usermanagement/1/user/group/direct', parameters, {'name': str(group_id)})
        
        return True
        
    def request_usernames_reminder(self, email):
        parameters = {'email': str(email)}
        
        self._perform_post('/rest/usermanagement/1/user/mail/usernames', parameters, {})
        
        return True
    
class ConfluenceInterface(AtlassianInterface):
    def __init__(self, credentials, location):
        AtlassianInterface.__init__(self, credentials, location)
        
    def get_space_permissions(self, space_key):
        result = self._perform_post('/rpc/json-rpc/confluenceservice-v2/getSpacePermissionSets', {}, [str(space_key)])

        if type(result.json()).__name__ == 'dict' and 'error' in result.json():
            raise helpers.OldAPIException('Error: the old API returned an error')
        else:
            found_space_permission_set = helpers.SpacePermissionSet()
            found_space_permission_set.load_json(result.json())
        
            return found_space_permission_set
        
    def get_space(self, space_key):
        result = self._perform_get('/rest/api/space/%s' % space_key, {'expand': 'description.plain'})
        
        found_space = helpers.SpaceObject()
        found_space.load_json(result.json())
        
        return found_space
    
    def create_space(self, space_key, name, description):
        result = self._perform_post('/rest/api/space', {}, {'key': str(space_key), 'name': str(name), 'description': {'plain': {'value': str(description), 'representation': 'plain'}}, 'metadata': {}})
        
        found_space = helpers.SpaceObject()
        found_space.load_json(result.json())
        
        return found_space
        
    def get_content(self, space_key, content_title):
        result = self._perform_get('/rest/api/content?spaceKey=%s&title=%s&expand=body.view,version,space' % (space_key, content_title), None)
        
        found_list = helpers.ContentListObject()
        found_list.load_json(result.json())
        
        return found_list.results
        
    def add_space_permissions(self, space_key, space_permission_addition):
        result = self._perform_post('/rpc/json-rpc/confluenceservice-v2/addPermissionsToSpace', {}, 
                                        [space_permission_addition.get_permission_vector(), 
                                         str(space_permission_addition.get_target()), str(space_key)])
        
        if type(result.json()).__name__ == 'dict' and 'error' in result.json():
            raise helpers.OldAPIException('Error: the old API returned an error')
        else:
            return True
            
class JiraInterface(AtlassianInterface):
    def __init__(self, credentials, location):
        AtlassianInterface.__init__(self, credentials, location)
        
    def search(self, query):
        result = self._perform_post('/rest/api/2/search', {}, {'jql': str(query)})
        
        found_issue_list = helpers.IssueListObject()
        found_issue_list.load_json(result.json())
        
        return found_issue_list
        
    def do_transition(self, issue_key, transition_id):
        self._perform_post('/rest/api/2/issue/%s/transitions' % issue_key, {}, {'transition': {'id': str(transition_id)}})
        
        return True
        
    def add_comment(self, issue_key, comment_text):
        self._perform_post('/rest/api/2/issue/%s/comment' % issue_key, {}, {'body': str(comment_text)})
        
        return True
        
    def update_issue_fields(self, issue_key, fields_dict):
        if type(fields_dict).__name__ != 'dict':
            raise helpers.IllegalArgumentsException('Fields to be updated should be in a dict!')
        elif len(fields_dict) == 0:
            raise helpers.IllegalArgumentsException('At least one field to be updated should be specified!')
            
        fields = {}
        for field in fields_dict:
            fields[field] = [{'set': fields_dict[field]}]
        
        self._perform_put('/rest/api/2/issue/%s' % issue_key, {}, {'update': fields})
        
        return True
    
    def customfield_add_option(self, customfield_id, option):
        self._perform_post('/rest/jiracustomfieldeditorplugin/1/user/customfields/%s/contexts/default/options' % customfield_id, 
                            {}, {'optionvalue': str(option), 'disabled': False})
        
        return True
    
    def customfield_sort_options(self, customfield_id):
        self._perform_put('/rest/jiracustomfieldeditorplugin/1/user/customfields/%s/contexts/default/options/sort' % customfield_id, 
                            {}, {'locale': 'en-US', 'order': 'ASCENDING'})
        
        return True
