#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#       
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#       
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json, logging
from os.path import expanduser

logger = logging.getLogger(__name__)

class AtlassianObject(object):
    def __init__(self, attributes):
        self._attributes = attributes;
        
        for attribute in self._attributes.keys():
            setattr(self, attribute, None);

    def load_json(self, json):
        if (logger.isEnabledFor(logging.DEBUG)):
            logger.debug('Got json representation: %s', json)

        for attribute in self._attributes.keys():
            json_search = self._attributes[attribute]
            
            if type(json_search).__name__ == 'str':
                json_search = [json_search]
                
            found_value = None
                
            if type(json_search).__name__ == 'list':
                found_value = self._load_attribute_from_json_recipe(json, json_search)
            elif type(json_search).__name__ == 'method':
                found_value = json_search(json)
            else:
                raise IllegalJsonRecognition('Could not understand the json search parameters specified in %s' % type(self).__name__)
                
            if logger.isEnabledFor(logging.DEBUG) and found_value:
                logger.debug('Assigning %s.%s: %s', type(self).__name__, attribute, found_value)
                
            setattr(self, attribute, found_value)
                
    def _load_attribute_from_json_recipe(self, json, json_search):
        current_branch = json
        
        for json_attribute in json_search:
            if type(current_branch).__name__ == 'dict' and json_attribute in current_branch:
                current_branch = current_branch[json_attribute]
            else:
                logger.warning('Assigning %s.%s: json representation does not contain %s', type(self).__name__, json_attribute, json_search)
                current_branch = None
        
        return current_branch
                
    def __str__(self):
        pretty = '<' + type(self).__name__
    
        for attribute in self._attributes.keys():
            pretty+= ' ' +attribute + '='
            
            value = getattr(self, attribute)
    
            if value:
                pretty+= '"' + str(value) + '"'
            else:
                pretty+= 'None'
    
        pretty+= '>'
    
        return pretty

class SpaceObject(AtlassianObject):
    _attributes = {'space_key': 'key', 
                    'name': 'name',
                    'description': ['description','plain','value']}

    def __init__(self):
        AtlassianObject.__init__(self, self._attributes)

class AtlassianAttributeObject(AtlassianObject):
    def __init__(self, attributes):
        attributes['attributes'] = self._list_attributes
        AtlassianObject.__init__(self, attributes)
    
    def _list_attributes(self, json):
        result_attributes = None
        
        if type(json).__name__ == 'dict' and 'attributes' in json:
            json = json['attributes']
            if json and 'attributes' in json:
                json = json['attributes']
                if type(json).__name__ == 'list':
                    result_attributes={}

                    for attribute in json:
                        if 'name' in attribute and 'values' in attribute:
                            values = attribute['values']
                            name = attribute['name']
                            
                            if type(name).__name__ == 'str' and type(values).__name__ == 'list' and len(values)>0:
                                if len(values) == 1:
                                    result_attributes[name] = values[0]
                                else:
                                    result_attributes[name] = values
                            else:
                                logger.warning('Parsing attributes: an attribute values is no list!')
                else:
                    logger.warning('Parsing attributes: the attributes are no list!')
                                
        return result_attributes

class UserObject(AtlassianAttributeObject):
    _attributes = {'first_name': 'first-name', 
                    'last_name': 'last-name', 
                    'display_name': 'display-name', 
                    'email': 'email', 
                    'active': 'active', 
                    'userid': 'name'}

    def __init__(self):
        AtlassianAttributeObject.__init__(self, self._attributes)

class EntityListObject(AtlassianObject):
    def __init__(self, list_attribute):   
        self._list_attribute = list_attribute
        AtlassianObject.__init__(self, {self._list_attribute: self._load_list})
        
    def _load_list(self, json):
        result_list = None
        
        if type(json).__name__ == 'dict' and self._list_attribute in json:
            json = json[self._list_attribute]
            
            if type(json).__name__ == 'list':
                result_list = []
                
                for user in json:
                    if type(user).__name__ == 'dict' and 'name' in user:
                        name = user['name']
                        
                        if type(name).__name__ == 'str':
                            result_list += [name]
                        else:
                            logger.warning('Parsing %s: user name is no string!', self._list_attribute)
            else:
                logger.warning('Parsing %s: the users are no list!', self._list_attribute)
                
        return result_list
        
class UserListObject(EntityListObject):
    def __init__(self):
        EntityListObject.__init__(self, 'users')

class GroupListObject(EntityListObject):
    def __init__(self):
        EntityListObject.__init__(self, 'groups')

class ContentListObject(AtlassianObject):
    def __init__(self):   
        self._list_attribute = 'results'
        AtlassianObject.__init__(self, {self._list_attribute: self._load_list})
        
    def _load_list(self, json):
        result_list = None
        
        if type(json).__name__ == 'dict' and self._list_attribute in json:
            json = json[self._list_attribute]
            
            if type(json).__name__ == 'list':
                result_list = []
                
                for content in json:
                    if type(content).__name__ == 'dict':
                        found_content = ContentObject()
                        found_content.load_json(content)
                        
                        result_list += [found_content]
            else:
                logger.warning('Parsing %s: the items are no list!', self._list_attribute)
                
        return result_list


class GroupObject(AtlassianAttributeObject):
    _attributes = {'group_id': 'name', 
                   'description': 'description', 
                   'active': 'active'}

    def __init__(self):
        AtlassianAttributeObject.__init__(self, self._attributes);

class IssueObject(AtlassianObject):
    _attributes = { 'issue_key': 'key', 
                    'issue_type': ['fields','issuetype','name'],
                    'project_key': ['fields','project','key'],
                    'status': ['fields','status','name'],
                    'summary': ['fields','summary'],
                    'fields': 'fields'}
                    
    def __init__(self):
        AtlassianObject.__init__(self, self._attributes)
        
    def __str__(self):
        pretty = '<' + type(self).__name__ + ' '
        
        pretty+= str(self.issue_type) + ' ' + str(self.issue_key)
        pretty+= ' summary="' + str(self.summary) + '">'
        
        return pretty

class IssueListObject:
    def __init__(self):
        self.issues = None;

    def load_json(self, json):
        if (logger.isEnabledFor(logging.DEBUG)):
            logger.debug('Got json representation: %s', json)
        
        self.issues = self._load_list(json)

    def _load_list(self, json):
        result_list = None
        
        if type(json).__name__ == 'dict' and 'issues' in json:
            json = json['issues']
            
            if type(json).__name__ == 'list':
                result_list = []
                
                for issue_json in json:
                    if type(issue_json).__name__ == 'dict':
                        issue = IssueObject()
                        issue.load_json(issue_json)
                        
                        result_list = result_list + [issue]
            else:
                logger.warning('Parsing IssueListObject: the issues are no list!')
                
        return result_list

    def __str__(self):
        pretty = '<' + type(self).__name__ + ' issues='
        
        if self.issues == None:
            pretty += 'None'
        else:
            pretty+= str([str(issue) for issue in self.issues]) + '>'
        
        return pretty

class SpacePermissionAddition(object):
    def __init__(self, target, is_target_a_group = True):
        self._target = target
        self._permission_vector = []
        self._is_target_a_group = is_target_a_group
        
    def is_target_a_group(self):
        return self._is_target_a_group
        
    def get_target(self):
        return self._target
        
    def get_permission_vector(self):
        return self._permission_vector
        
    def add_to_permission_vector(self, permission_name):
        self._permission_vector += [permission_name]
        
    def __str__(self):
        return '<SpacePermissionAddition target=\'' + str(self._target) + '\', target_is_group='+str(self._is_target_a_group)+', permission_vector=' + str(self._permission_vector) + ' >'


class SpacePermissionSet(object):
    def __init__(self):
        self._all_permissions = {}
    
        self._create_space_permission('_view_space', 'VIEWSPACE')
        self._create_space_permission('_delete_own', 'REMOVEOWNCONTENT')

        self._create_space_permission('_pages_add', 'EDITSPACE')
        self._create_space_permission('_pages_delete', 'REMOVEPAGE')
        
        self._create_space_permission('_blog_add', 'EDITBLOG')
        self._create_space_permission('_blog_delete', 'REMOVEBLOG')
        
        self._create_space_permission('_attachments_add', 'CREATEATTACHMENT')
        self._create_space_permission('_attachments_delete', 'REMOVEATTACHMENT')

        self._create_space_permission('_comments_add', 'COMMENT')
        self._create_space_permission('_comments_delete', 'REMOVECOMMENT')
        
        self._create_space_permission('_restrictions', 'SETPAGEPERMISSIONS')
        
        self._create_space_permission('_mail_delete', 'REMOVEMAIL')
        
        self._create_space_permission('_page_export', 'EXPORTPAGE')
        self._create_space_permission('_space_export', 'EXPORTSPACE')
        
        self._create_space_permission('_admin', 'SETSPACEPERMISSIONS')
        
    def _create_space_permission(self, attribute_name, permission_name):
        new_permission = SpacePermission(permission_name)
        
        self._all_permissions[permission_name] = new_permission
        
        def getter():
            return self._all_permissions[permission_name]
        
        def setter(groups, users):
            self._all_permissions[permission_name].set_groups(groups)
            self._all_permissions[permission_name].set_users(users)
        
        setattr(self, 'get%s' % attribute_name, getter)
        setattr(self, 'set%s_targets' % attribute_name, setter)
        
        return new_permission
        
    def load_json(self, json):
        if type(json).__name__ == 'list':
            for single_permission in json:
                if type(single_permission).__name__ == 'dict' and 'type' in single_permission:
                    single_permission_type = single_permission['type']
                    
                    if type(single_permission_type).__name__ == 'str' and single_permission_type in self._all_permissions:
                        self._all_permissions[single_permission_type].load_json(single_permission)
                    else:
                        logger.warning('Parsing space permissions set: the listed permission does not have a recognized type.')
                else:
                     logger.warning('Parsing space permissions set: the listed permission does not have a type.')
        else:
            logger.warning('Parsing space permissions set: not a list!')
    
    def get_permission_additions(self):
        target_dictionary = {}
        
        for permission_name, permission in sorted(self._all_permissions.items()):
            for target in permission.get_groups():
                if not target in target_dictionary:
                    target_dictionary[target] = SpacePermissionAddition(target)
                    
                target_dictionary[target].add_to_permission_vector(permission_name)
            
            for target in permission.get_users():
                if not target in target_dictionary:
                    target_dictionary[target] = SpacePermissionAddition(target, False)
                    
                target_dictionary[target].add_to_permission_vector(permission_name)
        
        return [target_dictionary[target] for target in sorted(target_dictionary)]
        
class SpacePermission(object):
    def __init__(self, permission_name):
        self._permission_name = permission_name
        self._users = []
        self._groups = []
        self._public = False
    
    def load_json(self, json):
        if type(json).__name__ == 'dict' and 'spacePermissions' in json:
            json = json['spacePermissions']
            
            if type(json).__name__ == 'list':
                for specification in json:
                    if type(specification).__name__ == 'dict' and 'type' in specification and\
                        'userName' in specification and 'groupName' in specification:
                        specification_type = specification['type']
                        if specification_type == self._permission_name:
                            user = specification['userName']
                            group = specification['groupName']
                            
                            if user != None:
                                self._users += [str(user)]
                            elif group != None:
                                self._groups += [str(group)]
                            else:
                                self._public = True
                        else:
                            logger.warning('Parsing spacePermissions.specification: wrong permission type!')
                    else:
                        logger.warning('Parsing spacePermissions.specification: not the expected format!')
            else:
                logger.warning('Parsing spacePermissions: not a list of permissions!')
        else:
            logger.warning('Parsing spacePermissions: not in the dictionary!')
    
    def get_users(self):
        return self._users
        
    def get_groups(self):
        return self._groups

    def set_users(self, users):
        self._users = users
        
    def set_groups(self, groups):
        self._groups = groups

    def get_public(self):
        return self._public

class ContentObject(AtlassianObject):
    _attributes = {'space_key': ['space', 'key'], 
                    'title': 'title',
                    'content_type': 'type',
                    'status': 'status',
                    'author': ['version', 'by', 'username'],
                    'last_changed': ['version', 'when'],
                    'body': ['body','view','value']}
    
    def __init__(self):
        AtlassianObject.__init__(self, self._attributes)
        
    def __str__(self):
        pretty = '<' + type(self).__name__ + ' '
        
        pretty+= str(self.content_type) + ' ' + str(self.space_key)
        pretty+= ' title="' + str(self.title)
        pretty+= '" status=' + str(self.status)
        pretty+= ' author=' + str(self.author)
        pretty+= ' last_changed=' + str(self.last_changed) + '>'
        
        return pretty

class Credentials(object):
    def __init__(self, username_field = 'username', password_field = 'password'):
        self._username = None
        self._password = None
        self._username_field = username_field
        self._password_field = password_field
        
    def __str__(self):
        return '<Credentials "' + (self._username or 'None') + '": *****>'
        
    def get_username(self):
        return self._username
        
    def get_password(self):
        return self._password
        
    def _load(self, json):
        if self._username_field in json:
            self._username = str(json[self._username_field])
            
        if self._password_field in json:
            self._password = str(json[self._password_field])

class CredentialsHolder(object):
    def __init__(self):
        self._loaded = False
        self._crowd_credentials = Credentials('application_username', 'application_password')
        self._confluence_credentials = Credentials()
        self._jira_credentials = Credentials()
        
    def __str__(self):
        return '<Credentials crowd=' + str(self._crowd_credentials) \
                + ', confluence=' + str(self._confluence_credentials) \
                + ', jira=' + str(self._jira_credentials)+ ' >'
        
    def get_crowd_credentials(self):
        if not self._loaded:
            self._load()
            
        return self._crowd_credentials

    def get_confluence_credentials(self):
        if not self._loaded:
            self._load()
            
        return self._confluence_credentials

    def get_jira_credentials(self):
        if not self._loaded:
            self._load()
            
        return self._jira_credentials

    def _load(self):
        self._loaded = True
        
        config_file = ConfigurationFiles().get_credentials_file()
        logger.info("Opening config file %s", config_file)
        
        with open(config_file, 'r') as config_file_open:
            data = json.load(config_file_open)
        
            if 'crowd' in data:
                self._crowd_credentials._load(data['crowd'])
            if 'confluence' in data:
                self._confluence_credentials._load(data['confluence'])
            if 'jira' in data:
                self._jira_credentials._load(data['jira'])
            
        logger.debug('Loaded Credentials: %s', self)
            
class Location(object):
    def __init__(self):
        self._url = None
    
    def __str__(self):
        return '<Location url="' + (self._url or 'None') + '">'
        
    def get_url(self):
        return self._url
        
    def _load(self, json):
        if 'url' in json:
            self._url = str(json['url'])
            
class LocationHolder(object):
    def __init__(self):
        self._loaded = False
        self._crowd_location = Location()
        self._confluence_location = Location()
        self._jira_location = Location()
        
    def __str__(self):
        return '<LocationHolder crowd=' + str(self._crowd_location) \
                + ', confluence=' + str(self._confluence_location) \
                + ', jira=' + str(self._jira_location)+ ' >'
        
    def get_crowd_location(self):
        if not self._loaded:
            self._load()
        return self._crowd_location
        
    def get_confluence_location(self):
        if not self._loaded:
            self._load()
        return self._confluence_location
        
    def get_jira_location(self):
        if not self._loaded:
            self._load()
        return self._jira_location
        
    def _load(self):
        self._loaded = True
        
        config_file = ConfigurationFiles().get_locations_file()
        logger.info("Opening config file %s", config_file)
        
        with open(config_file, 'r') as config_file_open:
            data = json.load(config_file_open)
            
            if 'crowd' in data:
                self._crowd_location._load(data['crowd'])
            if 'confluence' in data:
                self._confluence_location._load(data['confluence'])
            if 'jira' in data:
                self._jira_location._load(data['jira'])
            
        logger.debug('Loaded service location: %s', self)
            
class Singleton(object):
    _instance = None
    def __new__(class_, *args, **kwargs):
        if not isinstance(class_._instance, class_):
            class_._instance = object.__new__(class_, *args, **kwargs)
        return class_._instance

class ConfigurationFiles(Singleton, object):
    _base_directory = expanduser('~/.config/corporatesnorlax/')
    
    def set_base_directory(self, base_directory):
        self._base_directory = base_directory
        
        logger.info("ConfigurationFiles with base_directory %s", self._base_directory)
        
    def get_base_directory(self):
        return self._base_directory
        
    def get_credentials_file(self):
        return self._base_directory + 'credentials'
        
    def get_locations_file(self):
        return self._base_directory + 'locations'
        
class CacheHolder(object):
    def __init__(self):
        self._crowd_cache = CrowdCache()
        self._confluence_cache = ConfluenceCache()
        self._jira_cache = JiraCache()
        
    def get_crowd_cache(self):
        return self._crowd_cache
        
    def get_confluence_cache(self):
        return self._confluence_cache
        
    def get_jira_cache(self):
        return self._jira_cache
        
class CrowdCache(Singleton, object):
    def __init__(self):
        self._groups = {}
        self._groups_children = {}
        self._groups_parents = {}
        self._users = {}

    def set_crowd(self, crowd):
        self._crowd = crowd

    def get_group(self, group_id):
        if not group_id in self._groups:
            self._groups[group_id] = self._crowd.get_group(group_id)
            
        return self._groups[group_id]
        
    def get_child_groups_of_group(self, group_id):
        if not group_id in self._groups_children:
            self._groups_children[group_id] = self._crowd.get_child_groups_of_group(group_id)
            
        return self._groups_children[group_id]

    def get_parent_groups_of_group(self, group_id):
        if not group_id in self._groups_parents:
            self._groups_parents[group_id] = self._crowd.get_parent_groups_of_group(group_id)
            
        return self._groups_parents[group_id]
        
    def add_group(self, group_id, group_description):
        new_group = self._crowd.add_group(group_id, group_description)
        
        if new_group:
            self._groups[group_id] = new_group
            
        return new_group
        
    def add_direct_child_group_membership(self, parent_group_id, child_group_id):
        return self._crowd.add_direct_child_group_membership(parent_group_id, child_group_id)
        
    def get_user(self, user_id):
        if not user_id in self._users:
            self._users[user_id] = self._crowd.get_user(user_id)
        
        return self._users[user_id]
        
    def add_user(self, user_id, email, first_name, last_name, display_name, password):
        new_user = self._crowd.add_user(user_id, email, first_name, last_name, display_name, password)
        
        if new_user:
            self._users[user_id] = new_user
        
        return new_user
        
    def store_user_attributes(self, user_id, attributes_dict):
        self._users.pop(user_id, None)
        
        return self._crowd.store_user_attributes(user_id, attributes_dict)
        
    def add_user_to_group(self, user_id, group_id):
        return self._crowd.add_user_to_group(user_id, group_id)
        
    def request_usernames_reminder(self, email):
        return self._crowd.request_usernames_reminder(email)
        
class ConfluenceCache(Singleton, object):
    def __init__(self):
        self._spaces = {}
        self._spaces_permissions = {}

    def set_confluence(self, confluence):
        self._confluence = confluence

    def get_space_permissions(self, space_key):
        if not space_key in self._spaces_permissions:
            self._spaces_permissions[space_key] = self._confluence.get_space_permissions(space_key)
            
        return self._spaces_permissions[space_key]
        
    def get_space(self, space_key):
        if not space_key in self._spaces:
            self._spaces[space_key] = self._confluence.get_space(space_key)
            
        return self._spaces[space_key]
    
    def create_space(self, space_key, name, description):
        new_space = self._confluence.create_space(space_key, name, description)
        
        if new_space:
            self._spaces[space_key] = new_space
            
        return new_space
    
    def add_space_permissions(self, space_key, space_permission_addition):
        result = self._confluence.add_space_permissions(space_key, space_permission_addition)
            
        return result
    
    def get_content(self, space_key, content_title):
        result = self._confluence.get_content(space_key, content_title)
            
        return result
    
class JiraCache(Singleton, object):
    def __init__(self):
        pass

    def set_jira(self, jira):
        self._jira = jira
    
    def search(self, query):
        return self._jira.search(query)
        
    def do_transition(self, issue_key, transition_id):
        return self._jira.do_transition(issue_key, transition_id)
        
    def add_comment(self, issue_key, comment_text):
        return self._jira.add_comment(issue_key, comment_text)
    
    def update_issue_fields(self, issue_key, fields_dict):
        return self._jira.update_issue_fields(issue_key, fields_dict)
    
    def customfield_add_option(self, customfield_id, option):
        return self._jira.customfield_add_option(customfield_id, option)
    
    def customfield_sort_options(self, customfield_id):
        return self._jira.customfield_sort_options(customfield_id)
    
class AtlassianException(Exception):
    def __init__(self, message):
        self._message = message

class OldAPIException(AtlassianException):
    pass

class IllegalJsonRecognition(AtlassianException):
    pass
    
class UnauthorizedException(AtlassianException):
    pass
      
class ForbiddenException(AtlassianException):
    pass
        
class ObjectNotFoundException(AtlassianException):
    pass

class UnknownResponseException(AtlassianException):
    pass

class CrowdException(AtlassianException):
    pass
    
class IllegalRequestTypeException(AtlassianException):
    pass
      
class ObjectExistanceException(AtlassianException):
    pass
    
class IllegalArgumentsException(AtlassianException):
    pass
    
class FailedUpdateException(AtlassianException):
    pass
    
