#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#       
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#       
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from . import helpers
import requests, logging

logger = logging.getLogger(__name__)

class GroupExistance(object):
    def __init__(self, group_id):
        self._group_id = group_id

    def _check(self, cache_holder):
        logger.info('Checking that group %s exists', self._group_id)
        
        exists = False
        
        try:
            found = cache_holder.get_crowd_cache().get_group(self._group_id)
 
            logger.debug('Found group: %s', found)
            
            exists = found != None
        except helpers.ObjectNotFoundException:
            logger.debug('Got an object not found exception for group %s', self._group_id)
            
            exists = False

        return exists

class GroupExistanceRequirement(GroupExistance):
    def __init__(self, group_id, expect_existance):
        GroupExistance.__init__(self, group_id)
        self._expect_existance = bool(expect_existance)
        
    def __repr__(self):
        return self.__class__.__name__ + '(\'' + str(self._group_id) + '\', ' + str(self._expect_existance) + ')'

    def __eq__(self, other):
        return type(other).__name__ == self.__class__.__name__ and self._group_id == other._group_id and self._expect_existance and other._expect_existance

    def check_requirement(self, cache_holder):
        exists = self._check(cache_holder)

        logger.debug('Group exists: %s and required: %s', exists, self._expect_existance)
        
        return exists == self._expect_existance

class UserExistance(object):
    def __init__(self, user_id):
        self._user_id = user_id

    def _check(self, cache_holder):
        logger.info('Checking that user %s exists', self._user_id)
        
        exists = False
        
        try:
            found = cache_holder.get_crowd_cache().get_user(self._user_id)
 
            logger.debug('Found user: %s', found)
            
            exists = found != None
        except helpers.ObjectNotFoundException:
            logger.debug('Got an object not found exception for user %s', self._user_id)
            
            exists = False

        return exists

class UserExistanceRequirement(UserExistance):
    def __init__(self, user_id, expect_existance):
        UserExistance.__init__(self, user_id)
        self._expect_existance = bool(expect_existance)
        
    def __repr__(self):
        return self.__class__.__name__ + '(\'' + str(self._user_id) + '\', ' + str(self._expect_existance) + ')'

    def __eq__(self, other):
        return type(other).__name__ == self.__class__.__name__ and self._user_id == other._user_id and self._expect_existance and other._expect_existance

    def check_requirement(self, cache_holder):
        exists = self._check(cache_holder)

        logger.debug('User exists: %s and required: %s', exists, self._expect_existance)
        
        return exists == self._expect_existance

class SpaceExistance(object):
    def __init__(self, space_key):
        self._space_key = space_key

    def _check(self, cache_holder):
        logger.info('Checking that space %s exists', self._space_key)
        
        exists = False
        
        try:
            found = cache_holder.get_confluence_cache().get_space(self._space_key)
 
            logger.debug('Found space: %s', found)
            
            exists = found != None
        except helpers.ObjectNotFoundException:
            logger.debug('Got an object not found exception for space %s', self._space_key)
            
            exists = False

        return exists

class SpaceExistanceRequirement(SpaceExistance):
    def __init__(self, space_key, expect_existance):
        SpaceExistance.__init__(self, space_key)
        self._expect_existance = bool(expect_existance)
        
    def __repr__(self):
        return self.__class__.__name__ + '(\'' + str(self._space_key) + '\', ' + str(self._expect_existance) + ')'

    def __eq__(self, other):
        return type(other).__name__ == self.__class__.__name__ and self._space_key == other._space_key and self._expect_existance and other._expect_existance

    def check_requirement(self, cache_holder):
        exists = self._check(cache_holder)

        logger.debug('Space exists: %s and required: %s', exists, self._expect_existance)
        
        return exists == self._expect_existance

class StructureAction(object):
    def __init__(self):
        self._requirements = []
    
    def get_requirements(self):
        return self._requirements
    
    def get_satisfied_requirements_after_execution(self):
        return []
    
class CreateGroupAction(StructureAction):
    def __init__(self, group_id, group_description):
        StructureAction.__init__(self)
    
        self._group_id = group_id
        self._group_description = group_description
        
        self._requirements += [GroupExistanceRequirement(group_id, False)]
    
    def get_satisfied_requirements_after_execution(self):
        return [GroupExistanceRequirement(self._group_id, True)]
    
    def __repr__(self):
        return self.__class__.__name__ + '(\'' + str(self._group_id) + '\', \'' + str(self._group_description) + '\')'
    
    def run(self, cache_holder):
        result = False
        logger.info('Creating the group \'%s\' with description \'%s\'', self._group_id, self._group_description)
        
        try:
            new_group = cache_holder.get_crowd_cache().add_group(self._group_id, self._group_description)
            
            result = new_group != None
        except helpers.AtlassianException as atlassian_exception:
            logger.warning('Got an exception: %s', atlassian_exception)
            
        return result
    
class CreateUserAction(StructureAction):
    def __init__(self, user_id, email, first_name, last_name, display_name, password):
        StructureAction.__init__(self)
    
        self._user_id = user_id
        self._email = email
        self._first_name = first_name
        self._last_name = last_name
        self._display_name = display_name
        self._password = password
        
        self._requirements += [UserExistanceRequirement(user_id, False)]
    
    def get_satisfied_requirements_after_execution(self):
        return [UserExistanceRequirement(self._user_id, True)]
    
    def __repr__(self):
        return self.__class__.__name__ + '(\'' + str(self._user_id) + '\', \'' + str(self._email) + \
                '\', \'' + str(self._first_name) + '\', \'' + str(self._last_name) + \
                '\', \'' + str(self._display_name) + '\', \'***\')'
    
    def run(self, cache_holder):
        result = False
        logger.info('Creating the user \'%s\' with email \'%s\' and display name \'%s\'', self._user_id, \
                        self._email, self._display_name)
        
        try:
            new_user = cache_holder.get_crowd_cache().add_user(self._user_id, self._email, self._first_name, \
                    self._last_name, self._display_name, self._password)
            
            result = new_user != None
        except helpers.AtlassianException as atlassian_exception:
            logger.warning('Got an exception: %s', atlassian_exception)
            
        return result
    
class AddUserToGroupAction(StructureAction):
    def __init__(self, user_id, group_id):
        StructureAction.__init__(self)
    
        self._user_id = user_id
        self._group_id = group_id
        
        self._requirements += [UserExistanceRequirement(user_id, True),
                                GroupExistanceRequirement(group_id, True)]
    
    def __repr__(self):
        return self.__class__.__name__ + '(\'' + str(self._user_id) + '\', \'' + str(self._group_id) + '\')'
    
    def run(self, cache_holder):
        result = False
        logger.info('Adding the user \'%s\' to group \'%s\'', self._user_id, self._group_id)
        
        try:
            result = cache_holder.get_crowd_cache().add_user_to_group(self._user_id, self._group_id)
        except helpers.AtlassianException as atlassian_exception:
            logger.warning('Got an exception: %s', atlassian_exception)
            
        return result
    
class SetUserAttributesAction(StructureAction):
    def __init__(self, user_id, attributes):
        StructureAction.__init__(self)
    
        self._user_id = user_id
        self._attributes = attributes
        self._requirements += [UserExistanceRequirement(user_id, True)]
    
    def __repr__(self):
        return self.__class__.__name__ + '(\'' + str(self._user_id) + '\', ' + str(self._attributes) + ')'
    
    def run(self, cache_holder):
        result = False
        logger.info('Storing attributes to user \'%s\': %s', self._user_id, self._attributes)
        
        try:
            result = cache_holder.get_crowd_cache().store_user_attributes(self._user_id, self._attributes)
        except helpers.AtlassianException as atlassian_exception:
            logger.warning('Got an exception: %s', atlassian_exception)
            
        return result
    
class SendUsernameReminderAction(StructureAction):
    def __init__(self, user_id):
        StructureAction.__init__(self)
    
        self._user_id = user_id
        self._requirements += [UserExistanceRequirement(user_id, True)]
    
    def __repr__(self):
        return self.__class__.__name__ + '(\'' + str(self._user_id) + '\')'
    
    def run(self, cache_holder):
        result = False
        try:
            logger.info('Sending username reminder to user \'%s\': fetching user to recover email address', self._user_id)
            
            user = cache_holder.get_crowd_cache().get_user(self._user_id)
            
            if user and user.email:
                logger.info('Sending username reminder to user \'%s\': sending email', self._user_id)
                
                result = cache_holder.get_crowd_cache().request_usernames_reminder(user.email)
            else:
                logger.error('Could not fetch user \'%s\' to recover the email address', self._user_id)
        except helpers.AtlassianException as atlassian_exception:
            logger.warning('Got an exception: %s', atlassian_exception)
            
        return result
    
class CreateSpaceAction(StructureAction):
    def __init__(self, space_key, space_name, space_description):
        StructureAction.__init__(self)
    
        self._space_key = space_key
        self._space_name = space_name
        self._space_description = space_description
        
        self._requirements += [SpaceExistanceRequirement(space_key, False)]
    
    def get_satisfied_requirements_after_execution(self):
        return [SpaceExistanceRequirement(self._space_key, True)]
    
    def __repr__(self):
        return self.__class__.__name__ + '(\'' + str(self._space_key) + '\', \'' + str(self._space_name) + '\', \'' + str(self._space_description) + '\')'
    
    def run(self, cache_holder):
        result = False
        logger.info('Creating the space \'%s\' with name \'%s\' and description \'%s\'', self._space_key, self._space_name, self._space_description)
        
        try:
            new_space = cache_holder.get_confluence_cache().create_space(self._space_key, self._space_name, self._space_description)
            
            result = new_space != None
        except helpers.AtlassianException as atlassian_exception:
            logger.warning('Got an exception: %s', atlassian_exception)
            
        return result
        
class LinkGroupsAction(StructureAction):
    def __init__(self, parent_group_id, child_group_id):
        StructureAction.__init__(self)
    
        self._parent_group_id = parent_group_id
        self._child_group_id = child_group_id

        self._requirements += [GroupExistanceRequirement(parent_group_id, True), GroupExistanceRequirement(child_group_id, True)]

    def __repr__(self):
        return self.__class__.__name__ + '(\'' + str(self._parent_group_id) + '\', \'' + str(self._child_group_id) + '\')'

    def run(self, cache_holder):
        result = False
        logger.info('Creating the parent-child relationship between groups \'%s\' and \'%s\'', 
                    self._parent_group_id, self._child_group_id)
        
        try:
            result = cache_holder.get_crowd_cache().add_direct_child_group_membership(self._parent_group_id, self._child_group_id)
        except helpers.AtlassianException as atlassian_exception:
            logger.warning('Got an exception: %s', atlassian_exception)
            
        return result

class AddSpacePermissionsAction(StructureAction):
    def __init__(self, space_key, space_permission_addition):
        StructureAction.__init__(self)
    
        self._space_key = space_key
        self._space_permission_addition = space_permission_addition
        
        self._requirements += [SpaceExistanceRequirement(space_key, True)]
        if space_permission_addition.is_target_a_group():
            self._requirements += [GroupExistanceRequirement(space_permission_addition.get_target(), True)]
    
    def get_satisfied_requirements_after_execution(self):
        return []
    
    def __repr__(self):
        return self.__class__.__name__ + '(\'' + str(self._space_key) + '\', '+ str(self._space_permission_addition) + ')'
    
    def run(self, cache_holder):
        result = False
        logger.info('Adding the space permissions %s to the space \'%s\'', self._space_permission_addition, self._space_key)
        
        try:
            result = cache_holder.get_confluence_cache().add_space_permissions(self._space_key, self._space_permission_addition)
        except helpers.AtlassianException as atlassian_exception:
            logger.warning('Got an exception: %s', atlassian_exception)
            
        return result

class StructureUsecase(object):
    def __init__(self):
        self._action_list = []
        
    def _get_actions(self):
        return self._action_list
        
    def run(self, cache_holder, force_execution = False):
        result = False
        
        logger.info('Starting usecase %s: checking all requirements...', type(self).__name__)
        requirements = self.check_all_requirements(cache_holder)
        
        if requirements or force_execution:
            if requirements:
                logger.info('Requirements met, starting execution...')
            else:
                logger.warning('Requirements not completely met. Forcing execution nevertheless...')
                
            continue_execution = True
            failed_actions = []
            result = True
            
            for action in self._get_actions():
                if continue_execution:
                    logger.debug('Executing action: %s...', action)
                    action_result = action.run(cache_holder)
                
                    if not action_result:
                        failed_actions = failed_actions + [action]
                        logger.error('Failed action: %s.', action)
                        
                        if force_execution:
                            logger.warning('Forcing continuation of usecase %s even if the following actions failed: %s',
                                            type(self).__name__, failed_actions)
                        else:
                            logger.warning('Aborting usecase %s because of the failed action.', type(self).__name__)
                            
                            continue_execution = False
                            result = False
                else:
                    logger.debug('Skipping action: %s.', action)
        else:
            logger.error('Not all requirements for the usecase %s have been met, aborting.', type(self).__name__)
        
        return result
        
    def _get_all_requirements(self):
        requirements = []
        satisfied_requirements = []
        
        for action in self._get_actions():
            action_requirements = action.get_requirements()
            
            for requirement in action_requirements:
                if requirement not in satisfied_requirements and requirement not in requirements:
                    requirements += [requirement]
            
            satisfied_requirements += action.get_satisfied_requirements_after_execution()

        return requirements

    def check_all_requirements(self, cache_holder):
        requirements_satisfied = []
        requirements_unmet = []
        requirements = self._get_all_requirements()
        
        for requirement in requirements:
            if requirement.check_requirement(cache_holder):
                requirements_satisfied += [requirement]
            else:
                requirements_unmet += [requirement]
                
        result = len(requirements_satisfied) == len(requirements)
            
        logger.debug('Requirements: %s and satisfied: %s', requirements, requirements_satisfied)
        
        if result == False:
            logger.info('Requirements not satisfied: %s', requirements_unmet)
            
        return result

class CompositeStructureUsecase(StructureUsecase):
    def __init__(self, structure_usecase_list):
        StructureUsecase.__init__(self)
        self._structure_usecase_list = structure_usecase_list
        
    def _get_actions(self):
        total_actions = []
        
        for structure_usecase in self._structure_usecase_list:
            total_actions += structure_usecase._get_actions()
        
        return total_actions
        
    def run(self, cache_holder, force_execution = False):
        logger.info('Starting all the usecases in %s...', type(self).__name__)
        
        index=0
        result = True
        
        for index in range(0, len(self._structure_usecase_list)):
            if result == True:
                result = self.run_single(index, cache_holder, force_execution)
                
                if result == False:
                    logger.warning('Not executing any further usecase because of the failed usecase %s.', 
                                    type(self._structure_usecase_list[index]).__name__)
                                    
        return result
    
    def run_single(self, index, cache_holder, force_execution = False):
        return self._structure_usecase_list[index].run(cache_holder, force_execution)
    
