#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .context import corporatesnorlax
from corporatesnorlax import helpers
from .mockconfluence import MockConfluence
from . import mock

import unittest, logging, os

logger = logging.getLogger(__name__)

import requests, requests_mock

class TestWithMock(mock.BaseTestMock):
    def __init__(self, *args, **kwargs):
        mock.BaseTestMock.__init__(self, *args, **kwargs)
        
        self._credentials = self._all_credentials.get_confluence_credentials()
        
        self._mock_confluence_location = helpers.LocationHolder().get_confluence_location()
        self._mock_confluence = MockConfluence(self._mock_confluence_location)

class TestErrorCodes(TestWithMock):
    @requests_mock.Mocker()
    def test_unauthorized(self, mock):
        self._assert_error_code(mock, 'unauthorized', helpers.UnauthorizedException)
        
    @requests_mock.Mocker()
    def test_forbidden(self, mock):
        self._assert_error_code(mock, 'forbidden', helpers.ForbiddenException)
        
    @requests_mock.Mocker()
    def test_unknown(self, mock):
        self._assert_error_code(mock, 'servererror', helpers.UnknownResponseException)
        
    def _assert_error_code(self, mock, space_trigger, expected):
        self._mock_confluence.register_uris(mock)
    
        confluence = corporatesnorlax.ConfluenceInterface(self._credentials, self._mock_confluence_location)
    
        with self.assertRaises(expected):
            found = confluence.get_space(space_trigger)

            logger.error('Got %s', found)
            
class TestSpaces(TestWithMock):
    @requests_mock.Mocker()
    def test_get_space(self, mock):
        self._mock_confluence.register_uris(mock)
        
        confluence = corporatesnorlax.ConfluenceInterface(self._credentials, self._mock_confluence_location)
        
        space_object = confluence.get_space('existingspace')

        logger.debug('Got %s', space_object)

        self.assertEquals('existingspace', space_object.space_key)
        self.assertEquals('Space', space_object.name)
        self.assertEquals('Test Description', space_object.description)

    @requests_mock.Mocker()
    def test_get_space_invalid(self, mock):
        self._mock_confluence.register_uris(mock)
        
        confluence = corporatesnorlax.ConfluenceInterface(self._credentials, self._mock_confluence_location)
        
        with self.assertRaises(helpers.ObjectNotFoundException):
            space_object = confluence.get_space('invalidspace')

            logger.error('Got %s', space_object)
            
    @requests_mock.Mocker()
    def test_get_space_permissions(self, mock):
        self._mock_confluence.register_uris(mock)
        
        confluence = corporatesnorlax.ConfluenceInterface(self._credentials, self._mock_confluence_location)
        
        space_permission_set = confluence.get_space_permissions('existingspace')

        logger.debug('Got %s', space_permission_set)

        self.assertEquals(['group.orga'], space_permission_set.get_restrictions().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_restrictions().get_users())

        self.assertEquals(['group.orga'], space_permission_set.get_pages_delete().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_pages_delete().get_users())
        
        self.assertEquals(['group.orga'], space_permission_set.get_blog_add().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_blog_add().get_users())

        self.assertEquals([], space_permission_set.get_delete_own().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_delete_own().get_users())

        self.assertEquals(['group.orga', 'group.help'], space_permission_set.get_pages_add().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_pages_add().get_users())

        self.assertEquals([], space_permission_set.get_admin().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_admin().get_users())

        self.assertEquals([], space_permission_set.get_mail_delete().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_mail_delete().get_users())
        
        self.assertEquals(['jira-software-users', 'group.orga', 'confluence-users', 'group.help'], space_permission_set.get_view_space().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_view_space().get_users())

        self.assertEquals(['group.orga'], space_permission_set.get_blog_delete().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_blog_delete().get_users())

        self.assertEquals([], space_permission_set.get_page_export().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_page_export().get_users())

        self.assertEquals(['group.orga', 'confluence-users', 'group.help', 'jira-software-users'], space_permission_set.get_comments_add().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_comments_add().get_users())

        self.assertEquals(['group.orga', 'group.help'], space_permission_set.get_attachments_add().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_attachments_add().get_users())
        
        self.assertEquals(['group.orga'], space_permission_set.get_attachments_delete().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_attachments_delete().get_users())

        self.assertEquals(['group.orga'], space_permission_set.get_comments_delete().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_comments_delete().get_users())

        self.assertEquals([], space_permission_set.get_space_export().get_groups())
        self.assertEquals(['user.name'], space_permission_set.get_space_export().get_users())

    @requests_mock.Mocker()
    def test_get_space_permissions_invalid(self, mock):
        self._mock_confluence.register_uris(mock)
        
        confluence = corporatesnorlax.ConfluenceInterface(self._credentials, self._mock_confluence_location)
        
        with self.assertRaises(helpers.OldAPIException):
            space_permission_set = confluence.get_space_permissions('invalidspace')

            logger.debug('Got %s', space_permission_set)
            
    @requests_mock.Mocker()
    def test_create_space(self, mock):
        self._mock_confluence.register_uris(mock)
        
        confluence = corporatesnorlax.ConfluenceInterface(self._credentials, self._mock_confluence_location)
        
        space_object = confluence.create_space('NEW', 'New space', 'Test Description')

        logger.debug('Got %s', space_object)

        self.assertEquals('NEW', space_object.space_key)
        self.assertEquals('New space', space_object.name)
        self.assertEquals('Test Description', space_object.description)

    @requests_mock.Mocker()
    def test_create_space_existing(self, mock):
        self._mock_confluence.register_uris(mock)
        
        confluence = corporatesnorlax.ConfluenceInterface(self._credentials, self._mock_confluence_location)
        
        with self.assertRaises(helpers.ObjectExistanceException):
            space_object = confluence.create_space('EXISTING', 'Existing space', 'Test Description')

            logger.error('Got %s', space_object)
            
    @requests_mock.Mocker()
    def test_add_space_permissions(self, mock):
        self._mock_confluence.register_uris(mock)
        
        confluence = corporatesnorlax.ConfluenceInterface(self._credentials, self._mock_confluence_location)
        addition = helpers.SpacePermissionAddition('target')
        addition.add_to_permission_vector('NAME1')
        addition.add_to_permission_vector('NAME2')
        
        result = confluence.add_space_permissions('SPACE', addition)

        logger.debug('Got %s', result)

        self.assertEquals(True, result)

    @requests_mock.Mocker()
    def test_add_space_permissions_nonexisting(self, mock):
        self._mock_confluence.register_uris(mock)
        
        confluence = corporatesnorlax.ConfluenceInterface(self._credentials, self._mock_confluence_location)
        addition = helpers.SpacePermissionAddition('target')
        addition.add_to_permission_vector('NAME1')
        addition.add_to_permission_vector('NAME2')
        
        with self.assertRaises(helpers.OldAPIException):
            result = confluence.add_space_permissions('NONEX', addition)

            logger.error('Got %s', result)
            
    @requests_mock.Mocker()
    def test_add_space_permissions_notarget(self, mock):
        self._mock_confluence.register_uris(mock)
        
        confluence = corporatesnorlax.ConfluenceInterface(self._credentials, self._mock_confluence_location)
        addition = helpers.SpacePermissionAddition('nonexisting')
        addition.add_to_permission_vector('NAME1')
        addition.add_to_permission_vector('NAME2')
        
        with self.assertRaises(helpers.OldAPIException):
            result = confluence.add_space_permissions('SPACE', addition)

            logger.error('Got %s', result)
            
class TestContent(TestWithMock):
    @requests_mock.Mocker()
    def test_get_content(self, mock):
        self._mock_confluence.register_uris(mock)
        
        confluence = corporatesnorlax.ConfluenceInterface(self._credentials, self._mock_confluence_location)
        
        content_list = confluence.get_content('existingspace', 'Page Title')

        logger.debug('Got %s', content_list)

        self.assertEquals(1, len(content_list))
        
        content = content_list[0]
        
        self.assertEquals('existingspace', content.space_key)
        self.assertEquals('Page Title', content.title)
        self.assertEquals('page', content.content_type)
        self.assertEquals('current', content.status)
        self.assertEquals('user.name', content.author)
        self.assertEquals('<h1>Content</h1>', content.body)
        self.assertEquals('2017-09-12T22:06:24.556+02:00', content.last_changed)
        
        
