#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .context import corporatesnorlax
from corporatesnorlax import helpers
from .mockcrowd import MockCrowd
from . import mock

import unittest, logging

logger = logging.getLogger(__name__)

import requests, requests_mock

class MockAtlassianInterface(corporatesnorlax.AtlassianInterface):
    def __init__(self, credentials, base_url):
        corporatesnorlax.AtlassianInterface.__init__(self, credentials, base_url)

class TestWithMock(mock.BaseTestMock):
    def __init__(self, *args, **kwargs):
        mock.BaseTestMock.__init__(self, *args, **kwargs)
        
        self._credentials = self._all_credentials.get_crowd_credentials()
        
        self._mock_crowd_location = helpers.LocationHolder().get_crowd_location()
        self._mock_crowd = MockCrowd(self._mock_crowd_location)
        
class TestAtlassianInterface(TestWithMock):
    @requests_mock.Mocker()
    def test_unknown_method(self, mock):
        self._mock_crowd.register_uris(mock)
        
        interface = MockAtlassianInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.IllegalRequestTypeException):
            interface._perform_request('UNKNOWN', '/test/')
        
    @requests_mock.Mocker()
    def test_unknown_response_GET(self, mock):
        self._mock_crowd.register_uris(mock)
        
        interface = MockAtlassianInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.UnknownResponseException):
            interface._perform_request('GET', '/invalid-response/')
       
    @requests_mock.Mocker()
    def test_unknown_response_POST(self, mock):
        self._mock_crowd.register_uris(mock)
        
        interface = MockAtlassianInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.UnknownResponseException):
            interface._perform_request('POST', '/invalid-response/')
        
class TestErrorCodes(TestWithMock):
    @requests_mock.Mocker()
    def test_unauthorized(self, mock):
        self._assert_error_code(mock, 'unauthorized', helpers.UnauthorizedException)
        
    @requests_mock.Mocker()
    def test_forbidden(self, mock):
        self._assert_error_code(mock, 'forbidden', helpers.ForbiddenException)
        
    @requests_mock.Mocker()
    def test_unknown(self, mock):
        self._assert_error_code(mock, 'servererror', helpers.UnknownResponseException)
        
    def _assert_error_code(self, mock, user_trigger, expected):
        self._mock_crowd.register_uris(mock)
    
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
    
        with self.assertRaises(expected):
            found = crowd.get_user(user_trigger)

            logger.error('Got %s', found)

class TestUsers(TestWithMock):
    @requests_mock.Mocker()
    def test_get_user(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        user_object = crowd.get_user('existing.user')
        
        logger.debug('Got %s', user_object)
        
        self.assertEquals('existing.user', user_object.userid)
        self.assertEquals('Existing', user_object.first_name)
        self.assertEquals('User', user_object.last_name)
        self.assertEquals('Existing User', user_object.display_name)
        self.assertEquals('sample@user.cool', user_object.email)
        self.assertEquals(True, user_object.active)
        
        self.assertEquals({'invalidPasswordAttempts': '0', 'requiresPasswordChange': 'false'}, user_object.attributes)

    @requests_mock.Mocker()
    def test_get_user_invalid(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectNotFoundException):
            user_object = crowd.get_user('invalid.user')

            logger.error('Got %s', user_object)

    @requests_mock.Mocker()
    def test_add_user(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        user_object = crowd.add_user('new.user', 'sample@user.cool', 'New', 'User', 'New User', 'pass!')

        logger.debug('Got %s', user_object)

        self.assertEquals('new.user', user_object.userid)
        self.assertEquals('Existing', user_object.first_name)
        self.assertEquals('User', user_object.last_name)
        self.assertEquals('Existing User', user_object.display_name)
        self.assertEquals('sample@user.cool', user_object.email)
        self.assertEquals(True, user_object.active)

    @requests_mock.Mocker()
    def test_add_user_existing(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectExistanceException):
            user_object = crowd.add_user('existing.user', 'sample@user.cool', 'Existing', 'User', 'Existing User', 'password')

            logger.error('Got %s', user_object)

    @requests_mock.Mocker()
    def test_store_user_attributes(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        result = crowd.store_user_attributes('existing.user', {'otheratt': 'otherValue'})

        logger.debug('Got %s', result)
        
        self.assertEquals(True, result)

    @requests_mock.Mocker()
    def test_store_user_attributes_invalid_user(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd.store_user_attributes('invalid.user', {'attribute': 'value'})

            logger.error('Got %s', result)

    @requests_mock.Mocker()
    def test_store_user_attributes_invalid_attributes(self, mock):
        self._assert_store_user_attributes_invalid_attributes(mock, None)
        self._assert_store_user_attributes_invalid_attributes(mock, [])
        self._assert_store_user_attributes_invalid_attributes(mock, 10)
        self._assert_store_user_attributes_invalid_attributes(mock, 'Test')
        self._assert_store_user_attributes_invalid_attributes(mock, {})
        
    def _assert_store_user_attributes_invalid_attributes(self, mock, attributes):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.IllegalArgumentsException):
            result = crowd.store_user_attributes('any.user', attributes)

            logger.error('Got %s', result)

    @requests_mock.Mocker()
    def test_add_user_to_group(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        result = crowd.add_user_to_group('existing.user', 'existing.group')

        logger.debug('Got %s', result)
        
        self.assertEquals(True, result)

    @requests_mock.Mocker()
    def test_add_user_to_group_invalid_user(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd.add_user_to_group('invalid.user', 'existing.group')

            logger.error('Got %s', result)

    @requests_mock.Mocker()
    def test_add_user_to_group_invalid_group(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectExistanceException):
            result = crowd.add_user_to_group('existing.user', 'invalid.group')

            logger.error('Got %s', result)

    @requests_mock.Mocker()
    def test_add_user_to_group_already_there(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectExistanceException):
            result = crowd.add_user_to_group('connected.user', 'existing.group')

            logger.error('Got %s', result)

    @requests_mock.Mocker()
    def test_request_usernames_reminder(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        result = crowd.request_usernames_reminder('sample@user.cool')

        logger.debug('Got %s', result)

        self.assertEquals(True, result)

    @requests_mock.Mocker()
    def test_request_usernames_reminder_invalid(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectNotFoundException):
            user_object = crowd.request_usernames_reminder('invalid@mail.tld')

            logger.error('Got %s', user_object)

class TestGroups(TestWithMock):
    @requests_mock.Mocker()
    def test_get_group(self, mock):
        self._assert_get_group(mock, False)
        
    @requests_mock.Mocker()
    def test_get_group_expanded(self, mock):
        self._assert_get_group(mock, True)
        
    def _assert_get_group(self, mock, expanded):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        group_object = crowd.get_group('existing.group', expanded)

        logger.debug('Got %s', group_object)

        self.assertEquals('existing.group', group_object.group_id)
        self.assertEquals('Group Exists', group_object.description)
        self.assertEquals(True, group_object.active)
        
        if expanded:
            self.assertEquals({"testAttribute": "This is the value."}, group_object.attributes)
        else:
            self.assertEquals({}, group_object.attributes)
        
    @requests_mock.Mocker()
    def test_get_group_invalid(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectNotFoundException):
            user_object = crowd.get_group('invalid.group')

            logger.error('Got %s', group_object)
            
    @requests_mock.Mocker()
    def test_get_child_groups_of_group(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        group_list_object = crowd.get_child_groups_of_group('existing.group')

        logger.debug('Got %s', group_list_object)

        self.assertEquals(['childGroup1', 'childGroup2'], group_list_object.groups)
        
    @requests_mock.Mocker()
    def test_get_child_groups_of_group_invalid(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectNotFoundException):
            group_list_object = crowd.get_child_groups_of_group('invalid.group')

            logger.error('Got %s', group_list_object)
            
    @requests_mock.Mocker()
    def test_get_parent_groups_of_group(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        group_list_object = crowd.get_parent_groups_of_group('existing.group')

        logger.debug('Got %s', group_list_object)

        self.assertEquals(['parentGroup1', 'parentGroup2'], group_list_object.groups)
        
    @requests_mock.Mocker()
    def test_get_parent_groups_of_group_invalid(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectNotFoundException):
            group_list_object = crowd.get_parent_groups_of_group('invalid.group')

            logger.error('Got %s', group_list_object)
            
    @requests_mock.Mocker()
    def test_get_nested_members_of_group(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        user_list_object = crowd.get_nested_members_of_group('existing.group')

        logger.debug('Got %s', user_list_object)

        self.assertEquals(['sampleuser1', 'sampleuser2'], user_list_object.users)
        
    @requests_mock.Mocker()
    def test_get_nested_members_of_group_invalid(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectNotFoundException):
            user_list_object = crowd.get_nested_members_of_group('invalid.group')

            logger.error('Got %s', user_list_object)
            
    @requests_mock.Mocker()
    def test_add_group(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        group_object = crowd.add_group('new.group', 'Group Exists')

        logger.debug('Got %s', group_object)

        self.assertEquals('new.group', group_object.group_id)
        self.assertEquals('Group Exists', group_object.description)
        self.assertEquals(True, group_object.active)
        
        self.assertEquals({}, group_object.attributes)
            
    @requests_mock.Mocker()
    def test_add_group_existing(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectExistanceException):
            user_list_object = crowd.add_group('existing.group', 'Group Exists')

            logger.error('Got %s', user_list_object) 
            
    @requests_mock.Mocker()
    def test_add_direct_child_group_membership(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        result = crowd.add_direct_child_group_membership('parent.group', 'child.group')

        logger.debug('Got %s', result)
        
    @requests_mock.Mocker()
    def test_add_direct_child_group_membership_invalid_child(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectExistanceException):
            result = crowd.add_direct_child_group_membership('parent.group', 'invalid.group')

            logger.error('Got %s', result) 
            
    @requests_mock.Mocker()
    def test_add_direct_child_group_membership_invalid_parent(self, mock):
        self._mock_crowd.register_uris(mock)
        
        crowd = corporatesnorlax.CrowdInterface(self._credentials, self._mock_crowd_location)
        
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd.add_direct_child_group_membership('invalid.group', 'child.group')

            logger.error('Got %s', result) 
