#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .context import corporatesnorlax
from corporatesnorlax import helpers

import unittest, logging, os

logger = logging.getLogger(__name__)


class TestAtlassianObject(unittest.TestCase):
    class MockAtlassianObjectWithString(helpers.AtlassianObject):
        def __init__(self):
            helpers.AtlassianObject.__init__(self, {'string': 'value'})

    def test_attribute_string(self):
        mock = self.MockAtlassianObjectWithString()
        logger.debug('Created %s', mock)
        
        logger.debug('Attribute string %s', mock.string)
        self.assertIsNone(mock.string)
        
        mock.load_json({'value': 'found'})
        
        logger.debug('Attribute string %s', mock.string)
        self.assertEquals('found', mock.string)
        
        mock.load_json({'noattr': 'irrelevant'})
        
        logger.debug('Attribute string %s', mock.string)
        self.assertIsNone(mock.string)
        
    def test_attribute_string_negatives(self):
        self._assert_attribute_string_negatives(None)
        self._assert_attribute_string_negatives(1)
        self._assert_attribute_string_negatives('str')
        self._assert_attribute_string_negatives([])
        self._assert_attribute_string_negatives({})
        self._assert_attribute_string_negatives({'useless': 5})
        self._assert_attribute_string_negatives({'value': None})
        
    def _assert_attribute_string_negatives(self, json):
        mock = self.MockAtlassianObjectWithString()
        logger.debug('Created %s to test string_negative from %s', mock, json)
        
        mock.load_json(json)
        
        logger.debug('Attribute string %s', mock.string)
        self.assertIsNone(mock.string)
        
    class MockAtlassianObjectWithList(helpers.AtlassianObject):
        def __init__(self):
            helpers.AtlassianObject.__init__(self, {'list': ['first','second']})

    def test_attribute_list(self):
        mock = self.MockAtlassianObjectWithList()
        logger.debug('Created %s', mock)
        
        logger.debug('Attribute list %s', mock.list)
        self.assertIsNone(mock.list)
        
        mock.load_json({'first': {'second': 'found'}})
        
        logger.debug('Attribute list %s', mock.list)
        self.assertEquals('found', mock.list)
        
        mock.load_json({'first': {'third': 'found'}})
        
        logger.debug('Attribute list %s', mock.list)
        self.assertIsNone(mock.list)

    def test_attribute_list_negatives(self):
        self._assert_attribute_list_negatives(None)
        self._assert_attribute_list_negatives(1)
        self._assert_attribute_list_negatives('str')
        self._assert_attribute_list_negatives([])
        self._assert_attribute_list_negatives({})
        self._assert_attribute_list_negatives({'useless': 5})
        self._assert_attribute_list_negatives({'first': None})
        self._assert_attribute_list_negatives({'first': 5})
        self._assert_attribute_list_negatives({'first': 'Value'})
        self._assert_attribute_list_negatives({'first': []})
        self._assert_attribute_list_negatives({'first': {}})
        self._assert_attribute_list_negatives({'first': {'useless': 5}})
        self._assert_attribute_list_negatives({'first': {'first': 'value'}})
        self._assert_attribute_list_negatives({'first': {'second': None}})
        
    def _assert_attribute_list_negatives(self, json):
        mock = self.MockAtlassianObjectWithList()
        logger.debug('Created %s to test list_negative from %s', mock, json)
        
        mock.load_json(json)
        
        self.assertIsNone(mock.list)

    class MockAtlassianObjectWithFunction(helpers.AtlassianObject):
        def _callme(self, json):
            logger.debug('Called _callme with %s', json)
            
            self._json = json
            
            return self.return_value
    
        def __init__(self):
            helpers.AtlassianObject.__init__(self, {'function': self._callme})
            self._json = None
            self.return_value = 'first'

    def test_attribute_function(self):
        self._attribute_function(None)
        self._attribute_function(1)
        self._attribute_function('str')
        self._attribute_function([])
        self._attribute_function({})
        self._attribute_function({'useless': 5})
        self._attribute_function({'value': None})
    
    def _attribute_function(self, json):
        mock = self.MockAtlassianObjectWithFunction()
        logger.debug('Created %s to test list_function from %s', mock, json)
        
        mock.load_json(json)
        
        logger.debug('Attribute function %s', mock.function)
        self.assertEquals('first', mock.function)
        self.assertEquals(json, mock._json)
        
    class MockAtlassianObjectWithWrongType(helpers.AtlassianObject):
        def __init__(self):
            helpers.AtlassianObject.__init__(self, {'wrong': None})
            
    def test_attribute_wrong(self):
        mock = self.MockAtlassianObjectWithWrongType()
        logger.debug('Created %s to test exception', mock)
    
        with self.assertRaises(helpers.IllegalJsonRecognition):
            mock.load_json({})

            logger.error('Got %s', group_object)
    
class TestAtlassianAttributeObject(unittest.TestCase):
    def test_parse_invalid_object(self):
        self._assert_parse_returns_null_attributes(None)
        self._assert_parse_returns_null_attributes(1)
        self._assert_parse_returns_null_attributes('string')
        self._assert_parse_returns_null_attributes([])
        
    def test_parse_empty_object(self):
        self._assert_parse_returns_null_attributes({})
        
    def test_parse_empty_attribute_tag(self):
        self._assert_parse_returns_null_attributes({'attributes': {}})
        
    def test_parse_null_attribute_tag(self):
        self._assert_parse_returns_null_attributes({'attributes': None})
        
    def test_parse_empty_attribute_list(self):
        self._assert_parse_returns_null_attributes({'attributes': {'attributes': None}})
     
    def test_parse_attribute_list_not_list(self):
        self._assert_parse_returns_null_attributes({'attributes': {'attributes': 1}})
        self._assert_parse_returns_null_attributes({'attributes': {'attributes': {}}})
        self._assert_parse_returns_null_attributes({'attributes': {'attributes': 'str'}})
        
    def _assert_parse_returns_null_attributes(self, json):
        logger.debug('Asserting that parsing for attributes returns None: %s', json)
        test_object = helpers.AtlassianAttributeObject({})
        test_object.load_json(json)
        
        logger.info('Parsing json returned %s', test_object.attributes)
        self.assertIsNone(test_object.attributes)
        
    def test_parse_empty_attribute_list(self):
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': []}})
        
    def test_parse_attribute_list_empty_attribute(self):
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{}]}})

    def test_parse_attribute_list_no_names(self):
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'other': 'test'}]}})
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'name': None}]}})
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'values': ['1']}]}})
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'name': None, 'values': ['1']}]}})

    def test_parse_attribute_list_weird_name(self):
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'name': [], 'values': ['valid']}]}})
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'name': {}, 'values': ['valid']}]}})
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'name': ['t'], 'values': ['valid']}]}})
 
    def test_parse_attribute_list_weird_values(self):
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'name': 'valid'}]}})
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'name': 'valid', 'values': None}]}})
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'name': 'valid', 'values': {}}]}})
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'name': 'valid', 'values': 'string'}]}})
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'name': 'valid', 'values': 10}]}})
        self._assert_parse_returns_empty_attributes({'attributes': {'attributes': [{'name': 'valid', 'values': []}]}})

    def _assert_parse_returns_empty_attributes(self, json):
        logger.debug('Asserting that parsing for attributes returns an empty dictionary: %s', json)
        test_object = helpers.AtlassianAttributeObject({})
        test_object.load_json(json)
        
        logger.info('Parsing json returned %s', test_object.attributes)
        self.assertEquals({}, test_object.attributes)
        
    def test_parse_attribute_list(self):
            self._assert_parse_returns({'test1': True}, [ {"name": "test1","values": [True]},{"name": "test2","values": None},{"name": "test3","values": 'str'}])
            self._assert_parse_returns({'invalidPasswordAttempts': '0', 'requiresPasswordChange': 'false'}, [\
            {
                "link": {
                    "href": "https://crowdserver/crowd/rest/usermanagement/1/user/attributes?username=sampleuser&attributename=invalidPasswordAttempts",
                    "rel": "self"
                },
                "name": "invalidPasswordAttempts",
                "values": [
                    "0"
                ]
            },
            {
                "link": {
                    "href": "https://crowdserver/crowd/rest/usermanagement/1/user/attributes?username=sampleuser&attributename=invalidPasswordAttempts&attributename=requiresPasswordChange",
                    "rel": "self"
                },
                "name": "requiresPasswordChange",
                "values": [
                    "false"
                ]
            }
        ])
    
    def test_parse_attribute_list_multiple_values(self):
            self._assert_parse_returns({'test1': [True, 1], 'test2': 'str', 'test3': [0, 1, 2]}, [
                    {"name": "test1","values": [True, 1]},{"name": "test2","values": ['str']},{"name": "test3","values": [0, 1, 2]}
        ])
    
    def _assert_parse_returns(self, expected, json_attributes):
        json = {'attributes': {'attributes': json_attributes}}
        
        logger.debug('Asserting that parsing for attributes returns an empty dictionary: %s', json)
        test_object = helpers.AtlassianAttributeObject({})
        test_object.load_json(json)
        
        logger.info('Parsing json returned %s', test_object.attributes)
        self.assertEquals(expected, test_object.attributes)

class TestUserListObject(unittest.TestCase):
    def test_create(self):
        user_list_object = helpers.UserListObject()
        
        logger.debug('Got %s', user_list_object)

        self.assertIsNone(user_list_object.users)
        
    def test_load_complete(self):
        user_list_object = helpers.UserListObject()

        json_input = {'users': [{'name': 'user1'}, {'name': 'userTwo'}]}

        user_list_object.load_json(json_input)
        
        logger.debug('Got %s', user_list_object)

        self.assertEquals(['user1', 'userTwo'], user_list_object.users)

    def test_parse_wrong_objects(self):
        self._assert_users_none(None)
        self._assert_users_none('string')
        self._assert_users_none(1)
        self._assert_users_none([])

    def test_parse_empty_object(self):
        self._assert_users_none({})

    def test_parse_no_key(self):
        self._assert_users_none({'unnecessary': []})

    def test_parse_users_wrong_types(self):
        self._assert_users_none({'users': None})
        self._assert_users_none({'users': 'string'})
        self._assert_users_none({'users': 1})
        self._assert_users_none({'users': {}})
        
    def _assert_users_none(self, json):
        logger.debug('Asserting that parsing for users returns users=None: %s', json)
        user_list_object = helpers.UserListObject()
        user_list_object.load_json(json)
        
        logger.debug('Got %s', user_list_object)

        self.assertIsNone(user_list_object.users)
        
    def test_users_empty_list(self):
        self._assert_users_empty({'users': []})
        
    def test_users_list_wrong_objects(self):
        self._assert_users_empty({'users': [1, 2]})
        self._assert_users_empty({'users': ['string']})
        self._assert_users_empty({'users': [None]})
        
    def test_users_list_no_name(self):
        self._assert_users_empty({'users': [{}]})
        self._assert_users_empty({'users': [{'useless': 'test'}]})
        
    def test_users_list_name_wrong_type(self):
        self._assert_users_empty({'users': [{'name': None}]})
        self._assert_users_empty({'users': [{'name': 1}]})
        self._assert_users_empty({'users': [{'name': []}]})
        self._assert_users_empty({'users': [{'name': {}}]})
        
    def _assert_users_empty(self, json):
        logger.debug('Asserting that parsing for users returns an empty list: %s', json)
        user_list_object = helpers.UserListObject()
        user_list_object.load_json(json)
        
        logger.debug('Got %s', user_list_object)

        self.assertEquals([], user_list_object.users)
        
class TestGroupListObject(unittest.TestCase):
    def test_create(self):
        group_list_object = helpers.GroupListObject()
        
        logger.debug('Got %s', group_list_object)

        self.assertIsNone(group_list_object.groups)
        
    def test_load_complete(self):
        group_list_object = helpers.GroupListObject()

        json_input = {'groups': [{'name': 'group1'}, {'name': 'groupTwo'}]}

        group_list_object.load_json(json_input)
        
        logger.debug('Got %s', group_list_object)

        self.assertEquals(['group1', 'groupTwo'], group_list_object.groups)

    def test_parse_wrong_objects(self):
        self._assert_groups_none(None)
        self._assert_groups_none('string')
        self._assert_groups_none(1)
        self._assert_groups_none([])

    def test_parse_empty_object(self):
        self._assert_groups_none({})

    def test_parse_no_key(self):
        self._assert_groups_none({'unnecessary': []})

    def test_parse_groups_wrong_types(self):
        self._assert_groups_none({'groups': None})
        self._assert_groups_none({'groups': 'string'})
        self._assert_groups_none({'groups': 1})
        self._assert_groups_none({'groups': {}})
        
    def _assert_groups_none(self, json):
        logger.debug('Asserting that parsing for groups returns groups=None: %s', json)
        group_list_object = helpers.GroupListObject()
        group_list_object.load_json(json)
        
        logger.debug('Got %s', group_list_object)

        self.assertIsNone(group_list_object.groups)
        
    def test_groups_empty_list(self):
        self._assert_groups_empty({'groups': []})
        
    def test_groups_list_wrong_objects(self):
        self._assert_groups_empty({'groups': [1, 2]})
        self._assert_groups_empty({'groups': ['string']})
        self._assert_groups_empty({'groups': [None]})
        
    def test_groups_list_no_name(self):
        self._assert_groups_empty({'groups': [{}]})
        self._assert_groups_empty({'groups': [{'useless': 'test'}]})
        
    def test_groups_list_name_wrong_type(self):
        self._assert_groups_empty({'groups': [{'name': None}]})
        self._assert_groups_empty({'groups': [{'name': 1}]})
        self._assert_groups_empty({'groups': [{'name': []}]})
        self._assert_groups_empty({'groups': [{'name': {}}]})
        
    def _assert_groups_empty(self, json):
        logger.debug('Asserting that parsing for groups returns an empty list: %s', json)
        group_list_object = helpers.GroupListObject()
        group_list_object.load_json(json)
        
        logger.debug('Got %s', group_list_object)

        self.assertEquals([], group_list_object.groups)
        
class TestGroupObject(unittest.TestCase):
    def test_create(self):
        group_object = helpers.GroupObject()
        
        logger.debug('Got %s', group_object)

        self.assertIsNone(group_object.group_id)
        self.assertIsNone(group_object.description)
        self.assertIsNone(group_object.active)
        
    def test_load_complete(self):
        group_object = helpers.GroupObject()

        json_input = {'name': 'test.group', 'description': 'Test', 'active': True,
                      'attributes': {'attributes': [{'name': 'attribute1', 'values': ['one']},
                      {'name': 'attribute2', 'values': ['2', '3']}]}}

        group_object.load_json(json_input)
        
        logger.debug('Got %s', group_object)

        self.assertEquals(json_input['name'], group_object.group_id)
        self.assertEquals(json_input['description'], group_object.description)
        self.assertEquals(json_input['active'], group_object.active)
        self.assertEquals({'attribute1': 'one', 'attribute2': ['2', '3']}, group_object.attributes)

class TestUserObject(unittest.TestCase):
    def test_create(self):
        user_object = helpers.UserObject()
        
        logger.debug('Got %s', user_object)

        self.assertIsNone(user_object.userid)
        self.assertIsNone(user_object.first_name)
        self.assertIsNone(user_object.last_name)
        self.assertIsNone(user_object.display_name)
        self.assertIsNone(user_object.email)
        self.assertIsNone(user_object.active)
        
    def test_load_complete(self):
        user_object = helpers.UserObject()

        json_input = {'name': 'test.user', 'first-name': 'Test', 'last-name': 'User',
                      'display-name': 'User, Test', 'email': 'test@mail', 'active': True,
                      'attributes': {'attributes': [{'name': 'attribute1', 'values': ['one']},
                      {'name': 'attribute2', 'values': ['2', '3']}]}}

        user_object.load_json(json_input)
        
        logger.debug('Got %s', user_object)

        self.assertEquals(json_input['name'], user_object.userid)
        self.assertEquals(json_input['first-name'], user_object.first_name)
        self.assertEquals(json_input['last-name'], user_object.last_name)
        self.assertEquals(json_input['display-name'], user_object.display_name)
        self.assertEquals(json_input['email'], user_object.email)
        self.assertEquals(json_input['active'], user_object.active)
        self.assertEquals({'attribute1': 'one', 'attribute2': ['2', '3']}, user_object.attributes)

class TestContentObject(unittest.TestCase):
    def test_create(self):
        content_object = helpers.ContentObject()
        
        logger.debug('Got %s', content_object)

        self.assertIsNone(content_object.space_key)
        self.assertIsNone(content_object.title)
        self.assertIsNone(content_object.content_type)
        self.assertIsNone(content_object.status)
        self.assertIsNone(content_object.author)
        self.assertIsNone(content_object.last_changed)
        self.assertIsNone(content_object.body)

    def test_load_complete(self):
        content_object = helpers.ContentObject()

        json_input = {'type': 'page', 'status': 'current', 'title': 'My page title', 
                        'space': {'key': 'KEY'}, 'version': {'by': {'username': 'user.name'}, 
                        'when': 'timestamp'}, 'body': {'view': {'value': 'HTML'}}}

        content_object.load_json(json_input)
        
        logger.debug('Got %s', content_object)

        self.assertEquals(json_input['type'], content_object.content_type)
        self.assertEquals(json_input['status'], content_object.status)        
        self.assertEquals(json_input['title'], content_object.title)
        self.assertEquals(json_input['space']['key'], content_object.space_key)
        self.assertEquals(json_input['version']['by']['username'], content_object.author)
        self.assertEquals(json_input['version']['when'], content_object.last_changed)
        self.assertEquals(json_input['body']['view']['value'], content_object.body)

    def test_load_list_complete(self):
        content_list_object = helpers.ContentListObject()

        json_input = {'type': 'page', 'status': 'current', 'title': 'My page title', 
                        'space': {'key': 'KEY'}, 'version': {'by': {'username': 'user.name'}, 
                        'when': 'timestamp'}, 'body': {'view': {'value': 'HTML'}}}
        json_input_list = {'results': [json_input]}

        content_list_object.load_json(json_input_list)
        
        logger.debug('Got %s', content_list_object)

        self.assertEquals(1, len(content_list_object.results))
        content_object = content_list_object.results[0]

        self.assertEquals(json_input['type'], content_object.content_type)
        self.assertEquals(json_input['status'], content_object.status)        
        self.assertEquals(json_input['title'], content_object.title)
        self.assertEquals(json_input['space']['key'], content_object.space_key)
        self.assertEquals(json_input['version']['by']['username'], content_object.author)
        self.assertEquals(json_input['version']['when'], content_object.last_changed)
        self.assertEquals(json_input['body']['view']['value'], content_object.body)
        
    def test_load_list_empty(self):
        self._assert_load_list_noresults({'results': []}, False)
        
    def test_load_list_noresults(self):
        self._assert_load_list_noresults({})
        
    def test_load_list_wrongtype(self):
        self._assert_load_list_noresults({'results': None})
        self._assert_load_list_noresults({'results': {}})
        self._assert_load_list_noresults({'results': 134})
        
    def _assert_load_list_noresults(self, json_input_list, isNone=True):
        content_list_object = helpers.ContentListObject()

        content_list_object.load_json(json_input_list)
        
        logger.debug('Got %s', content_list_object)
        
        if isNone:
            self.assertIsNone(content_list_object.results)
        else:
            self.assertEquals(0, len(content_list_object.results))
        
class TestIssueObject(unittest.TestCase):
    def test_create(self):
        issue_object = helpers.IssueObject()
        
        logger.debug('Got %s', issue_object)

        self.assertIsNone(issue_object.issue_key)
        self.assertIsNone(issue_object.summary)
        self.assertIsNone(issue_object.issue_type)
        self.assertIsNone(issue_object.project_key)
        self.assertIsNone(issue_object.status)
        self.assertIsNone(issue_object.fields)
        
    def test_load_complete(self):
        issue_object = helpers.IssueObject()
        fields = {'customfield_10002': None, 'progress': {'progress': 0, 'total': 0}, 'summary': 'summary!', \
                    'status': {'name': 'To Do'}, 'project': {'key': 'PP'}, 'issuetype': {'name': 'Epic'}}

        json_input = {'id': '10001', 'fields': fields, 'key': 'PP-8'}

        issue_object.load_json(json_input)
        
        logger.debug('Got %s', issue_object)

        self.assertEquals(json_input['key'], issue_object.issue_key)
        self.assertEquals(fields, issue_object.fields)
        self.assertEquals(fields['summary'], issue_object.summary)
        self.assertEquals(fields['project']['key'], issue_object.project_key)
        self.assertEquals(fields['status']['name'], issue_object.status)
        self.assertEquals(fields['issuetype']['name'], issue_object.issue_type)

class TestIssueListObject(unittest.TestCase):
    def test_create(self):
        issue_list_object = helpers.IssueListObject()
        
        logger.debug('Got %s', issue_list_object)

        self.assertIsNone(issue_list_object.issues)
        
    def test_load_complete(self):
        issue_list_object = helpers.IssueListObject()
        fields = {'customfield_10002': None, 'progress': {'progress': 0, 'total': 0}, \
                    'status': {'name': 'To Do'}, 'project': {'key': 'PP'}, 'issuetype': {'name': 'Epic'}}

        json_input = {'issues': [{'id': '10001', 'fields': fields, 'key': 'PP-1'},
                                {'id': '10002', 'fields': fields, 'key': 'PP-2'},
                                {'id': '10003', 'fields': fields, 'key': 'PP-3'},
                                {'id': '10004', 'fields': fields, 'key': 'PP-4'},
                                {'id': '10005', 'fields': fields, 'key': 'PP-5'}]}

        issue_list_object.load_json(json_input)
        
        logger.debug('Got %s', issue_list_object)

        self.assertEquals(len(json_input['issues']), len(issue_list_object.issues))
        self.assertEquals('PP-1', issue_list_object.issues[0].issue_key)
        self.assertEquals('PP-2', issue_list_object.issues[1].issue_key)
        
    def test_load_json_no_dict(self):
        self._assert_load_empty_issues(None)
        self._assert_load_empty_issues([])
        self._assert_load_empty_issues(1)
        self._assert_load_empty_issues('tst')

    def test_load_json_no_dict_key(self):
        self._assert_load_empty_issues({})
        self._assert_load_empty_issues({'key': 'wrong'})
        self._assert_load_empty_issues({10: 'issues'})

    def test_load_json_list_no_list(self):
        self._assert_load_empty_issues({'issues': None})
        self._assert_load_empty_issues({'issues': 5})
        self._assert_load_empty_issues({'issues': 'Test'})
        self._assert_load_empty_issues({'issues': {}})
        
    def _assert_load_empty_issues(self, json):
        issue_list_object = helpers.IssueListObject()
        
        issue_list_object.load_json(json)
        
        self.assertIsNone(issue_list_object.issues)
        
class TestSpaceObject(unittest.TestCase):
    def test_create(self):
        space_object = helpers.SpaceObject()
        
        logger.debug('Got %s', space_object)

        self.assertIsNone(space_object.space_key)
        self.assertIsNone(space_object.name)
        self.assertIsNone(space_object.description)
        
    def test_load_complete(self):
        space_object = helpers.SpaceObject()

        json_input = {'key': 'testKEY', 'type': 'global', 'name': 'Test Name', 'id': '019293', 'description':{'plain': {'value': 'description', 'representation': 'plain'}, 'view': {}}}

        space_object.load_json(json_input)
        
        logger.debug('Got %s', space_object)

        self.assertEquals(json_input['key'], space_object.space_key)
        self.assertEquals(json_input['name'], space_object.name)
        self.assertEquals(json_input['description']['plain']['value'], space_object.description)

class TestSpacePermissionAddition(unittest.TestCase):
    def test_create(self):
        permission_addition = helpers.SpacePermissionAddition('target')
        
        self.assertEquals('target', permission_addition.get_target())
        self.assertEquals([], permission_addition.get_permission_vector())

    def test_add_permissions(self):
        permission_addition = helpers.SpacePermissionAddition('target')
        
        permission_addition.add_to_permission_vector('EDITBLOG')
        permission_addition.add_to_permission_vector('COMMENT')
        permission_addition.add_to_permission_vector('REMOVEMAIL')
        
        self.assertEquals('target', permission_addition.get_target())
        self.assertEquals(['EDITBLOG', 'COMMENT', 'REMOVEMAIL'], permission_addition.get_permission_vector())

class TestSpacePermissionSet(unittest.TestCase):
    def test_create(self):
        permission_set = helpers.SpacePermissionSet();
        
        self._assert_empty_permissions(permission_set)
        
    def _assert_empty_permissions(self, permission_set):
        self._assert_permissions([],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[],[], 
                                permission_set)
        
    def _assert_permissions(self, expect_view_space_groups, expect_view_space_users, expect_delete_own_groups, \
            expect_delete_own_users, expect_pages_add_groups, expect_pages_add_users, expect_pages_delete_groups, \
            expect_pages_delete_users, expect_blog_add_groups, expect_blog_add_users, expect_blog_delete_groups, \
            expect_blog_delete_users, expect_attachments_add_groups, expect_attachments_add_users, \
            expect_attachments_delete_groups, expect_attachments_delete_users, expect_comments_add_groups, \
            expect_comments_add_users, expect_comments_delete_groups, expect_comments_delete_users, \
            expect_restrictions_groups, expect_restrictions_users, expect_mail_delete_groups, \
            expect_mail_delete_users, expect_page_export_groups, expect_page_export_users, \
            expect_space_export_groups, expect_space_export_users, expect_admin_groups, expect_admin_users, \
            permission_set):
        self._assert_permission(expect_view_space_groups, expect_view_space_users, permission_set.get_view_space())
        self._assert_permission(expect_delete_own_groups, expect_delete_own_users,permission_set.get_delete_own())

        self._assert_permission(expect_pages_add_groups, expect_pages_add_users, permission_set.get_pages_add())
        self._assert_permission(expect_pages_delete_groups, expect_pages_delete_users, permission_set.get_pages_delete())
        
        self._assert_permission(expect_blog_add_groups, expect_blog_add_users, permission_set.get_blog_add())
        self._assert_permission(expect_blog_delete_groups, expect_blog_delete_users, permission_set.get_blog_delete())
        
        self._assert_permission(expect_attachments_add_groups, expect_attachments_add_users, permission_set.get_attachments_add())
        self._assert_permission(expect_attachments_delete_groups, expect_attachments_delete_users, permission_set.get_attachments_delete())

        self._assert_permission(expect_comments_add_groups, expect_comments_add_users, permission_set.get_comments_add())
        self._assert_permission(expect_comments_delete_groups, expect_comments_delete_users, permission_set.get_comments_delete())
        
        self._assert_permission(expect_restrictions_groups, expect_restrictions_users, permission_set.get_restrictions())
        
        self._assert_permission(expect_mail_delete_groups, expect_mail_delete_users, permission_set.get_mail_delete())
        
        self._assert_permission(expect_page_export_groups, expect_page_export_users, permission_set.get_page_export())
        self._assert_permission(expect_space_export_groups, expect_space_export_users, permission_set.get_space_export())
        
        self._assert_permission(expect_admin_groups, expect_admin_users, permission_set.get_admin())
        
    def _assert_permission(self, expect_groups, expect_users, permission):
        self.assertEquals(expect_groups, permission.get_groups())
        self.assertEquals(expect_users, permission.get_users())

    def test_load_json_example(self):
        permission_set = helpers.SpacePermissionSet()
        
        logger.debug('Loading JSON')
        
        permission_set.load_json([{'type': 'VIEWSPACE', 'spacePermissions': [{'type': 'VIEWSPACE', 'userName': None, 'groupName': 'view_space_group'},{'type': 'VIEWSPACE', 'userName': 'view_space_user', 'groupName': None}]},
            {'type': 'REMOVEOWNCONTENT', 'spacePermissions': [{'type': 'REMOVEOWNCONTENT', 'userName': None, 'groupName': 'delete_own_group'},{'type': 'REMOVEOWNCONTENT', 'userName': 'delete_own_user', 'groupName': None}]},
            {'type': 'EDITSPACE', 'spacePermissions': [{'type': 'EDITSPACE', 'userName': None, 'groupName': 'pages_add_group'},{'type': 'EDITSPACE', 'userName': 'pages_add_user', 'groupName': None}]},
            {'type': 'REMOVEPAGE', 'spacePermissions': [{'type': 'REMOVEPAGE', 'userName': None, 'groupName': 'pages_delete_group'},{'type': 'REMOVEPAGE', 'userName': 'pages_delete_user', 'groupName': None}]},
            {'type': 'EDITBLOG', 'spacePermissions': [{'type': 'EDITBLOG', 'userName': None, 'groupName': 'blog_add_group'},{'type': 'EDITBLOG', 'userName': 'blog_add_user', 'groupName': None}]},
            {'type': 'REMOVEBLOG', 'spacePermissions': [{'type': 'REMOVEBLOG', 'userName': None, 'groupName': 'blog_delete_group'},{'type': 'REMOVEBLOG', 'userName': 'blog_delete_user', 'groupName': None}]},
            {'type': 'CREATEATTACHMENT', 'spacePermissions': [{'type': 'CREATEATTACHMENT', 'userName': None, 'groupName': 'attachments_add_group'},{'type': 'CREATEATTACHMENT', 'userName': 'attachments_add_user', 'groupName': None}]},
            {'type': 'REMOVEATTACHMENT', 'spacePermissions': [{'type': 'REMOVEATTACHMENT', 'userName': None, 'groupName': 'attachments_delete_group'},{'type': 'REMOVEATTACHMENT', 'userName': 'attachments_delete_user', 'groupName': None}]},
            {'type': 'COMMENT', 'spacePermissions': [{'type': 'COMMENT', 'userName': None, 'groupName': 'comments_add_group'},{'type': 'COMMENT', 'userName': 'comments_add_user', 'groupName': None}]},
            {'type': 'REMOVECOMMENT', 'spacePermissions': [{'type': 'REMOVECOMMENT', 'userName': None, 'groupName': 'comments_delete_group'},{'type': 'REMOVECOMMENT', 'userName': 'comments_delete_user', 'groupName': None}]},
            {'type': 'SETPAGEPERMISSIONS', 'spacePermissions': [{'type': 'SETPAGEPERMISSIONS', 'userName': None, 'groupName': 'restrictions_group'},{'type': 'SETPAGEPERMISSIONS', 'userName': 'restrictions_user', 'groupName': None}]},
            {'type': 'REMOVEMAIL', 'spacePermissions': [{'type': 'REMOVEMAIL', 'userName': None, 'groupName': 'mail_delete_group'},{'type': 'REMOVEMAIL', 'userName': 'mail_delete_user', 'groupName': None}]},
            {'type': 'EXPORTPAGE', 'spacePermissions': [{'type': 'EXPORTPAGE', 'userName': None, 'groupName': 'page_export_group'},{'type': 'EXPORTPAGE', 'userName': 'page_export_user', 'groupName': None}]},
            {'type': 'EXPORTSPACE', 'spacePermissions': [{'type': 'EXPORTSPACE', 'userName': None, 'groupName': 'space_export_group'},{'type': 'EXPORTSPACE', 'userName': 'space_export_user', 'groupName': None}]},
            {'type': 'SETSPACEPERMISSIONS', 'spacePermissions': [{'type': 'SETSPACEPERMISSIONS', 'userName': None, 'groupName': 'admin_group'},{'type': 'SETSPACEPERMISSIONS', 'userName': 'admin_user', 'groupName': None}]}])
        
        logger.debug('Got %s, asserting permissions', permission_set)
        
        self._assert_permissions(['view_space_group'], ['view_space_user'], ['delete_own_group'], ['delete_own_user'], \
            ['pages_add_group'], ['pages_add_user'], ['pages_delete_group'], ['pages_delete_user'], ['blog_add_group'], \
            ['blog_add_user'], ['blog_delete_group'], ['blog_delete_user'], ['attachments_add_group'], ['attachments_add_user'], \
            ['attachments_delete_group'], ['attachments_delete_user'], ['comments_add_group'], ['comments_add_user'], \
            ['comments_delete_group'], ['comments_delete_user'], ['restrictions_group'], ['restrictions_user'], \
            ['mail_delete_group'], ['mail_delete_user'], ['page_export_group'], ['page_export_user'], ['space_export_group'], \
            ['space_export_user'], ['admin_group'], ['admin_user'], permission_set)
    
    def test_set_permissions_example(self):
        permission_set = helpers.SpacePermissionSet()
        
        logger.debug('Adding permissions manually')
        
        permission_set.set_view_space_targets(['view_space_group'], ['view_space_user'])
        permission_set.set_delete_own_targets(['delete_own_group'], ['delete_own_user'])

        permission_set.set_pages_add_targets(['pages_add_group'], ['pages_add_user'])
        permission_set.set_pages_delete_targets(['pages_delete_group'], ['pages_delete_user'])
        
        permission_set.set_blog_add_targets(['blog_add_group'], ['blog_add_user'])
        permission_set.set_blog_delete_targets(['blog_delete_group'], ['blog_delete_user'])
        
        permission_set.set_attachments_add_targets(['attachments_add_group'], ['attachments_add_user'])
        permission_set.set_attachments_delete_targets(['attachments_delete_group'], ['attachments_delete_user'])

        permission_set.set_comments_add_targets(['comments_add_group'], ['comments_add_user'])
        permission_set.set_comments_delete_targets(['comments_delete_group'], ['comments_delete_user'])
        
        permission_set.set_restrictions_targets(['restrictions_group'], ['restrictions_user'])
        
        permission_set.set_mail_delete_targets(['mail_delete_group'], ['mail_delete_user'])
        
        permission_set.set_page_export_targets(['page_export_group'], ['page_export_user'])
        permission_set.set_space_export_targets(['space_export_group'], ['space_export_user'])
        
        permission_set.set_admin_targets(['admin_group'], ['admin_user'])
        
        logger.debug('Got %s, asserting permissions', permission_set)
        
        self._assert_permissions(['view_space_group'], ['view_space_user'], ['delete_own_group'], ['delete_own_user'], \
            ['pages_add_group'], ['pages_add_user'], ['pages_delete_group'], ['pages_delete_user'], ['blog_add_group'], \
            ['blog_add_user'], ['blog_delete_group'], ['blog_delete_user'], ['attachments_add_group'], ['attachments_add_user'], \
            ['attachments_delete_group'], ['attachments_delete_user'], ['comments_add_group'], ['comments_add_user'], \
            ['comments_delete_group'], ['comments_delete_user'], ['restrictions_group'], ['restrictions_user'], \
            ['mail_delete_group'], ['mail_delete_user'], ['page_export_group'], ['page_export_user'], ['space_export_group'], \
            ['space_export_user'], ['admin_group'], ['admin_user'], permission_set)
    
    def test_get_permissions_addition(self):
        permission_set = helpers.SpacePermissionSet()
        
        logger.debug('Adding permissions manually')
        
        permission_set.set_view_space_targets(['standard_group', 'privileged_group'], ['admin'])
        permission_set.set_delete_own_targets(['standard_group', 'privileged_group'], ['admin'])

        permission_set.set_pages_add_targets(['privileged_group'], ['admin'])
        permission_set.set_pages_delete_targets(['privileged_group'], ['admin'])
        
        permission_set.set_blog_add_targets(['privileged_group'], ['admin'])
        permission_set.set_blog_delete_targets(['privileged_group'], ['admin'])
        
        permission_set.set_attachments_add_targets(['privileged_group'], ['admin'])
        permission_set.set_attachments_delete_targets(['privileged_group'], ['admin'])

        permission_set.set_comments_add_targets(['standard_group', 'privileged_group'], ['admin'])
        permission_set.set_comments_delete_targets(['privileged_group'], ['admin'])
        
        permission_set.set_restrictions_targets(['privileged_group'], ['admin'])
        
        permission_set.set_mail_delete_targets([], ['admin'])
        
        permission_set.set_page_export_targets([], ['admin'])
        permission_set.set_space_export_targets([], ['admin'])
        
        permission_set.set_admin_targets([], ['admin'])
        
        additions = permission_set.get_permission_additions()
        
        logger.debug('Got additions %s, checking them', additions)
        self.assertEquals('<SpacePermissionAddition target=\'admin\', target_is_group=False, permission_vector=[\'COMMENT\', \'CREATEATTACHMENT\', \'EDITBLOG\', \'EDITSPACE\', \'EXPORTPAGE\', \'EXPORTSPACE\', \'REMOVEATTACHMENT\', \'REMOVEBLOG\', \'REMOVECOMMENT\', \'REMOVEMAIL\', \'REMOVEOWNCONTENT\', \'REMOVEPAGE\', \'SETPAGEPERMISSIONS\', \'SETSPACEPERMISSIONS\', \'VIEWSPACE\'] >', str(additions[0]))
        self.assertEquals('<SpacePermissionAddition target=\'privileged_group\', target_is_group=True, permission_vector=[\'COMMENT\', \'CREATEATTACHMENT\', \'EDITBLOG\', \'EDITSPACE\', \'REMOVEATTACHMENT\', \'REMOVEBLOG\', \'REMOVECOMMENT\', \'REMOVEOWNCONTENT\', \'REMOVEPAGE\', \'SETPAGEPERMISSIONS\', \'VIEWSPACE\'] >', str(additions[1]))
        self.assertEquals('<SpacePermissionAddition target=\'standard_group\', target_is_group=True, permission_vector=[\'COMMENT\', \'REMOVEOWNCONTENT\', \'VIEWSPACE\'] >', str(additions[2]))

    def test_load_json_no_list(self):
        self._assert_load_empty_permissions(None)
        self._assert_load_empty_permissions({})
        self._assert_load_empty_permissions(1)
        self._assert_load_empty_permissions('tst')

    def test_load_json_no_list_of_dicts(self):
        self._assert_load_empty_permissions([None])
        self._assert_load_empty_permissions([[]])
        self._assert_load_empty_permissions([1])
        self._assert_load_empty_permissions(['tst'])

    def test_load_json_no_list_of_dicts_with_type(self):
        self._assert_load_empty_permissions([{'test': 1}])
        self._assert_load_empty_permissions([{'spacePermissions': {}}])

    def test_load_json_list_of_dicts_invalid_type(self):
        self._assert_load_empty_permissions([{'spacePermissions': {}, 'type': 'UNKNOWN'}])
        self._assert_load_empty_permissions([{'spacePermissions': {}, 'type': None}])
        self._assert_load_empty_permissions([{'spacePermissions': {}, 'type': 1}])
        self._assert_load_empty_permissions([{'spacePermissions': {}, 'type': [1,2,3]}])
        
    def _assert_load_empty_permissions(self, json):
        permission_set = helpers.SpacePermissionSet();
        
        permission_set.load_json(json)
        
        self._assert_empty_permissions(permission_set)

class TestSpacePermission(unittest.TestCase):
    def test_create(self):
        permission = helpers.SpacePermission('NAME');
        
        self.assertEquals([], permission.get_groups())
        self.assertEquals([], permission.get_users())
        self.assertEquals(False, permission.get_public())
        
    def test_load_json_user(self):
        permission = helpers.SpacePermission('NAME');
        
        permission.load_json({'type': 'NAME','spacePermissions':[{'type': 'NAME','userName': 'user5','groupName': None}]})
        
        self.assertEquals([], permission.get_groups())
        self.assertEquals(['user5'], permission.get_users())
        self.assertEquals(False, permission.get_public())
        
    def test_load_json_group(self):
        permission = helpers.SpacePermission('NAME');
        
        permission.load_json({'type': 'NAME','spacePermissions':[{'type': 'NAME','userName': None,'groupName': 'group3'}]})
        
        self.assertEquals(['group3'], permission.get_groups())
        self.assertEquals([], permission.get_users())
        self.assertEquals(False, permission.get_public())
        
    def test_load_json_multiple_1(self):
        permission = helpers.SpacePermission('NAME');
        
        permission.load_json({'type': 'NAME','spacePermissions':[{'type': 'NAME','userName': 'user8','groupName': None},{'type': 'NAME','userName': None,'groupName': None},{'type': 'NAME','userName': None,'groupName': 'group4'}]})
        
        self.assertEquals(['group4'], permission.get_groups())
        self.assertEquals(['user8'], permission.get_users())
        self.assertEquals(True, permission.get_public())
        
    def test_load_json_multiple_2(self):
        permission = helpers.SpacePermission('NAME');
        
        permission.load_json({'type': 'NAME','spacePermissions':[{'type': 'NAME','userName': 'user6','groupName': None},{'type': 'NAME','userName': None,'groupName': 'group9'},{'type': 'NAME','userName': 'user1','groupName': None}]})
        
        self.assertEquals(['group9'], permission.get_groups())
        self.assertEquals(['user6','user1'], permission.get_users())
        self.assertEquals(False, permission.get_public())

    def test_load_json_else_nodict(self):
        self.assert_load_json_empty(None)
        self.assert_load_json_empty("Dict")
        self.assert_load_json_empty(1)
        self.assert_load_json_empty([])
        
    def test_load_json_else_nokey(self):
        self.assert_load_json_empty({})
        self.assert_load_json_empty({'test': '10'})

    def test_load_json_else_nolist(self):
        self.assert_load_json_empty({'spacePermissions': None})
        self.assert_load_json_empty({'spacePermissions': 10})
        self.assert_load_json_empty({'spacePermissions': 'Test'})
        self.assert_load_json_empty({'spacePermissions': {}})

    def test_load_json_else_spec_nodict(self):
        self.assert_load_json_empty({'spacePermissions': [None]})
        self.assert_load_json_empty({'spacePermissions': [10]})
        self.assert_load_json_empty({'spacePermissions': ['list']})
        self.assert_load_json_empty({'spacePermissions': [[]]})
        
    def test_load_json_else_spec_nokeys(self):
        self.assert_load_json_empty({'spacePermissions': [{'test': True}]})
        self.assert_load_json_empty({'spacePermissions': [{'type': 'NAME'}]})
        self.assert_load_json_empty({'spacePermissions': [{'type': 'NAME', 'userName': 'test'}]})
        self.assert_load_json_empty({'spacePermissions': [{'groupName': None, 'userName': 'test'}]})
        self.assert_load_json_empty({'spacePermissions': [{'groupName': 'test', 'type': 'NAME'}]})

    def test_load_json_else_spec_wrongtype(self):
        self.assert_load_json_empty({'spacePermissions': [{'groupName': 'test', 'type': 'NAME2', 'userName': None}]})
        self.assert_load_json_empty({'spacePermissions': [{'groupName': 'test', 'type': None, 'userName': None}]})
        self.assert_load_json_empty({'spacePermissions': [{'groupName': 'test', 'type': [], 'userName': None}]})
        self.assert_load_json_empty({'spacePermissions': [{'groupName': 'test', 'type': {}, 'userName': None}]})
        
    def test_load_json_else_spec_allnone(self):
        self.assert_load_json_empty({'spacePermissions': [{'groupName': None, 'type': 'NAME', 'userName': None}]}, True)
        
    def assert_load_json_empty(self, json, public = False):
        permission = helpers.SpacePermission('NAME');
        
        permission.load_json(json)
        
        self.assertEquals([], permission.get_groups())
        self.assertEquals([], permission.get_users())
        self.assertEquals(public, permission.get_public())

class ConfigurationTestCase(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        unittest.TestCase.__init__(self, *args, **kwargs)
        configuration_files = helpers.ConfigurationFiles()
        configuration_files.set_base_directory(os.path.dirname(os.path.realpath(__file__)) + '/config/')
        
        logger.debug('Initialized configuration_files with base directory %s', configuration_files.get_base_directory())

class TestCredentialsHolder(ConfigurationTestCase):
    def test_get_crowd_credentials(self):
        credentials_holder = helpers.CredentialsHolder()
        crowd_credentials = credentials_holder.get_crowd_credentials()
        
        application_username = crowd_credentials.get_username()
        logger.debug('Got application username "%s"', application_username)
        self.assertEquals('chosen-username', application_username)
        
        application_password = crowd_credentials.get_password()
        logger.debug('Got application password "%s"', application_password)
        self.assertEquals('hunter2', application_password)
        
    def test_get_crowd_credentials_andersrum(self):
        credentials_holder = helpers.CredentialsHolder()
        crowd_credentials = credentials_holder.get_crowd_credentials()
        
        application_password = crowd_credentials.get_password()
        logger.debug('Got application password "%s"', application_password)
        self.assertEquals('hunter2', application_password)
        
        application_username = crowd_credentials.get_username()
        logger.debug('Got application username "%s"', application_username)
        self.assertEquals('chosen-username', application_username)
        
    def test_get_confluence_credentials(self):
        credentials_holder = helpers.CredentialsHolder()
        confluence_credentials = credentials_holder.get_confluence_credentials()
        
        username = confluence_credentials.get_username()
        logger.debug('Got username "%s"', username)
        self.assertEquals('user.id', username)
        
        password = confluence_credentials.get_password()
        logger.debug('Got password "%s"', password)
        self.assertEquals('halftheanswer', password)
        
    def test_get_confluence_credentials_andersrum(self):
        credentials_holder = helpers.CredentialsHolder()
        confluence_credentials = credentials_holder.get_confluence_credentials()

        password = confluence_credentials.get_password()
        logger.debug('Got password "%s"', password)
        self.assertEquals('halftheanswer', password)

        username = confluence_credentials.get_username()
        logger.debug('Got username "%s"', username)
        self.assertEquals('user.id', username)
        
    def test_get_jira_credentials(self):
        credentials_holder = helpers.CredentialsHolder()
        jira_credentials = credentials_holder.get_jira_credentials()
        
        username = jira_credentials.get_username()
        logger.debug('Got username "%s"', username)
        self.assertEquals('jiragod', username)
        
        password = jira_credentials.get_password()
        logger.debug('Got password "%s"', password)
        self.assertEquals('fourwordsalluppercase', password)
        
    def test_get_jira_credentials_andersrum(self):
        credentials_holder = helpers.CredentialsHolder()
        jira_credentials = credentials_holder.get_jira_credentials()

        password = jira_credentials.get_password()
        logger.debug('Got password "%s"', password)
        self.assertEquals('fourwordsalluppercase', password)

        username = jira_credentials.get_username()
        logger.debug('Got username "%s"', username)
        self.assertEquals('jiragod', username)
        
class TestLocationHolder(ConfigurationTestCase):
    def test_load_crowd_location(self):
        location = helpers.LocationHolder()
        
        self.assertEquals('https://crowd-url.tld/c', location.get_crowd_location().get_url())
        
    def test_load_confluence_location(self):
        location = helpers.LocationHolder()
        
        self.assertEquals('https://confluence-url.tld/c', location.get_confluence_location().get_url())
        
    def test_load_jira_location(self):
        location = helpers.LocationHolder()
        
        self.assertEquals('https://jira-url.tld/j', location.get_jira_location().get_url())
        
class TestCrowdCache(unittest.TestCase):
    def test_get_from_cache_holder(self):
        holder = helpers.CacheHolder()
        
        logger.debug('Expecting that the object from the CacheHolder is the same as a new one')
        
        mock_crowd = MockCrowdCache([], [])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(holder.get_crowd_cache(), crowd_cache)
        self.assertEquals(holder.get_crowd_cache()._crowd, crowd_cache._crowd)
        
    def test_get_group(self):
        test_group_name = 'test.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance')
        mock_crowd = MockCrowdCache([test_group_name],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Getting a test group and expecting the mock to receive the call')
        result = crowd_cache.get_group(test_group_name)
        
        self.assertEquals(test_group_name, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting the same test group and expecting the result to be in cache')
        result = crowd_cache.get_group(test_group_name)
        
        self.assertEquals(test_group_name, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
    def test_get_group_not_existing(self):
        test_group_name = 'nonexisting.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no groups')
        mock_crowd = MockCrowdCache([],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Getting a nonexisting group and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.get_group(test_group_name)
        
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting a nonexisting group and expecting the request not to be cached')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.get_group(test_group_name)
        
        self.assertEquals(2, mock_crowd.get_hits())
        
    def test_get_child_groups_of_group(self):
        test_group_name = 'test.group'
        test_group_child = test_group_name + '_child_1'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance')
        mock_crowd = MockCrowdCache([test_group_name, test_group_child],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Getting child groups and expecting the mock to receive the call')
        result = crowd_cache.get_child_groups_of_group(test_group_name)
        
        self.assertEquals([test_group_child], result)
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting child groups of the same test group and expecting the result to be in cache')
        result = crowd_cache.get_child_groups_of_group(test_group_name)
        
        self.assertEquals([test_group_child], result)
        self.assertEquals(1, mock_crowd.get_hits())
        
    def test_get_child_groups_of_group_not_existing(self):
        test_group_name = 'nonexisting.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no groups')
        mock_crowd = MockCrowdCache([],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Getting child groups of a nonexisting group and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.get_child_groups_of_group(test_group_name)
        
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting child groups of a nonexisting group and expecting the request not to be cached')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.get_child_groups_of_group(test_group_name)
        
        self.assertEquals(2, mock_crowd.get_hits())
        
    def test_get_parent_groups_of_group(self):
        test_group_name = 'test.group'
        test_group_parent = test_group_name + '_parent_1'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance')
        mock_crowd = MockCrowdCache([test_group_name, test_group_parent],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Getting parent groups and expecting the mock to receive the call')
        result = crowd_cache.get_parent_groups_of_group(test_group_name)
        
        self.assertEquals([test_group_parent], result)
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting parent groups of the same test group and expecting the result to be in cache')
        result = crowd_cache.get_parent_groups_of_group(test_group_name)
        
        self.assertEquals([test_group_parent], result)
        self.assertEquals(1, mock_crowd.get_hits())
        
    def test_get_parent_groups_of_group_not_existing(self):
        test_group_name = 'nonexisting.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no groups')
        mock_crowd = MockCrowdCache([],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Getting parent groups of a nonexisting group and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.get_parent_groups_of_group(test_group_name)
        
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting parent groups of a nonexisting group and expecting the request not to be cached')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.get_parent_groups_of_group(test_group_name)
        
        self.assertEquals(2, mock_crowd.get_hits())
        
    def test_add_group(self):
        test_group_name = 'test.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no groups')
        mock_crowd = MockCrowdCache([],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Adding a group and expecting the mock to receive the call')
        result = crowd_cache.add_group(test_group_name, 'Test Description')
        
        self.assertEquals(test_group_name, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting the same group and expecting the request to be cached')
        result = crowd_cache.get_group(test_group_name)
        
        self.assertEquals(test_group_name, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
    def test_add_group_existing(self):
        test_group_name = 'existing.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance')
        mock_crowd = MockCrowdCache([test_group_name],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Adding an existing group and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectExistanceException):
            result = crowd_cache.add_group(test_group_name, 'Test Description')
        
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting the same group and expecting the mock to receive the call')
        result = crowd_cache.get_group(test_group_name)
        
        self.assertEquals(test_group_name, result)
        self.assertEquals(2, mock_crowd.get_hits())

    def test_add_direct_child_group_membership(self):
        test_parent_group_name = 'parent.group'
        test_child_group_name = 'child.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance')
        mock_crowd = MockCrowdCache([test_parent_group_name, test_child_group_name],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Adding a direct child group membership and expecting the mock to receive the call')
        result = crowd_cache.add_direct_child_group_membership(test_parent_group_name, test_child_group_name)
        
        self.assertEquals(True, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
    def test_add_direct_child_group_membership_parent_non_existing(self):
        test_parent_group_name = 'nonexisting.group'
        test_child_group_name = 'child.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no parent')
        mock_crowd = MockCrowdCache([test_child_group_name],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Adding a direct child group membership and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.add_direct_child_group_membership(test_parent_group_name, test_child_group_name)
        
        self.assertEquals(1, mock_crowd.get_hits())
        
    def test_add_direct_child_group_membership_child_non_existing(self):
        test_parent_group_name = 'parent.group'
        test_child_group_name = 'nonexisting.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no child')
        mock_crowd = MockCrowdCache([test_parent_group_name],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Adding a direct child group membership and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectExistanceException):
            result = crowd_cache.add_direct_child_group_membership(test_parent_group_name, test_child_group_name)
        
        self.assertEquals(1, mock_crowd.get_hits())
        
    def test_add_user(self):
        test_user_name = 'test.user'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no users')
        mock_crowd = MockCrowdCache([],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Adding a user and expecting the mock to receive the call')
        result = crowd_cache.add_user(test_user_name, 'test@mail.tld', 'Test', 'User', 'Test User', 'PWD')
        
        self.assertEquals(test_user_name, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting the same user and expecting the request to be cached')
        result = crowd_cache.get_user(test_user_name)
        
        self.assertEquals(test_user_name, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
    def test_add_user_existing(self):
        test_user_name = 'existing.user'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance')
        mock_crowd = MockCrowdCache([],[test_user_name])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Adding an existing user and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectExistanceException):
            result = crowd_cache.add_user(test_user_name, 'test@mail.tld', 'Test', 'User', 'Test User', 'PWD')
        
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting the same user and expecting the mock to receive the call')
        result = crowd_cache.get_user(test_user_name)
        
        self.assertEquals(test_user_name, result)
        self.assertEquals(2, mock_crowd.get_hits())
        
    def test_get_user(self):
        test_user_name = 'test.user'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance')
        mock_crowd = MockCrowdCache([],[test_user_name])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Getting a test user and expecting the mock to receive the call')
        result = crowd_cache.get_user(test_user_name)
        
        self.assertEquals(test_user_name, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting the same test user and expecting the result to be in cache')
        result = crowd_cache.get_user(test_user_name)
        
        self.assertEquals(test_user_name, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
    def test_get_user_not_existing(self):
        test_user_name = 'nonexisting.user'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no users')
        mock_crowd = MockCrowdCache([],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Getting a nonexisting user and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.get_user(test_user_name)
        
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Getting a nonexisting user and expecting the request not to be cached')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.get_user(test_user_name)
        
        self.assertEquals(2, mock_crowd.get_hits())
        
    def test_store_user_attributes(self):
        test_user_name = 'test.user'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, one user')
        mock_crowd = MockCrowdCache([],[test_user_name])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Getting the same user and expecting the request to be cached')
        result = crowd_cache.get_user(test_user_name)
        
        self.assertEquals(test_user_name, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Storing a user attribute and expecting the mock to receive the call, deleting the cache')
        result = crowd_cache.store_user_attributes(test_user_name, {'attribute': 'value'})
        
        self.assertEquals(True, result)
        self.assertEquals(2, mock_crowd.get_hits())
        
        logger.debug('Getting the same user and expecting the request not to be in cache')
        result = crowd_cache.get_user(test_user_name)
        
        self.assertEquals(test_user_name, result)
        self.assertEquals(3, mock_crowd.get_hits())
        
    def test_store_user_attributes_notexisting(self):
        test_user_name = 'nonexisting.user'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no users')
        mock_crowd = MockCrowdCache([],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Storing a user attribute for a nonexisting user and expecting the request not to be cached')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.store_user_attributes(test_user_name, {'attribute': 'value'})
        
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Storing a user attribute for a nonexisting user again, and expecting the request not to be cached')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.store_user_attributes(test_user_name, {'attribute': 'value'})
        
        self.assertEquals(2, mock_crowd.get_hits())
        
    def test_add_user_to_group(self):
        test_user_name = 'test.user'
        test_group_name = 'test.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, one user, one group')
        mock_crowd = MockCrowdCache([test_group_name],[test_user_name])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Adding the user to the group and expecting the mock to receive the call.')
        result = crowd_cache.add_user_to_group(test_user_name, test_group_name)
        
        self.assertEquals(True, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Adding the user to the group again and expecting the mock to receive the call.')
        result = crowd_cache.add_user_to_group(test_user_name, test_group_name)
        
        self.assertEquals(True, result)
        self.assertEquals(2, mock_crowd.get_hits())
        
    def test_add_inexistent_user_to_group(self):
        test_user_name = 'inexistent.user'
        test_group_name = 'test.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no user, one group')
        mock_crowd = MockCrowdCache([test_group_name],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Adding the user to the group and expecting the mock to receive the call.')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.add_user_to_group(test_user_name, test_group_name)
        
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Adding the user to the group again and expecting the mock to receive the call.')
        with self.assertRaises(helpers.ObjectNotFoundException):
                result = crowd_cache.add_user_to_group(test_user_name, test_group_name)
        
        self.assertEquals(2, mock_crowd.get_hits())
        
    def test_add_user_to_inexistent_group(self):
        test_user_name = 'test.user'
        test_group_name = 'inexistent.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, one user, no groups')
        mock_crowd = MockCrowdCache([],[test_user_name])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Adding the user to the group and expecting the mock to receive the call.')
        with self.assertRaises(helpers.ObjectExistanceException):
            result = crowd_cache.add_user_to_group(test_user_name, test_group_name)
        
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Adding the user to the group again and expecting the mock to receive the call.')
        with self.assertRaises(helpers.ObjectExistanceException):
            result = crowd_cache.add_user_to_group(test_user_name, test_group_name)
        
        self.assertEquals(2, mock_crowd.get_hits())
        
    def test_add_inexistent_user_to_inexistent_group(self):
        test_user_name = 'inexistent.user'
        test_group_name = 'inexistent.group'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no user, no group')
        mock_crowd = MockCrowdCache([],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Adding the user to the group and expecting the mock to receive the call.')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.add_user_to_group(test_user_name, test_group_name)
        
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Adding the user to the group again and expecting the mock to receive the call.')
        with self.assertRaises(helpers.ObjectNotFoundException):
                result = crowd_cache.add_user_to_group(test_user_name, test_group_name)
        
        self.assertEquals(2, mock_crowd.get_hits())
        
    def test_request_usernames_reminder(self):
        test_email = 'test@mail.tld'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, one user+email')
        mock_crowd = MockCrowdCache([],[test_email])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Requesting a reminder mail and expecting the mock to receive the call.')
        result = crowd_cache.request_usernames_reminder(test_email)
        
        self.assertEquals(True, result)
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Requesting a reminder mail and expecting the mock to receive the call.')
        result = crowd_cache.request_usernames_reminder(test_email)
        
        self.assertEquals(True, result)
        self.assertEquals(2, mock_crowd.get_hits())
        
    def test_request_usernames_reminder_invalid(self):
        test_email = 'invalid@mail.tld'
        
        logger.debug('Initializing a crowd cache with a mock as crowd instance, no users')
        mock_crowd = MockCrowdCache([],[])
        crowd_cache = helpers.CrowdCache()
        crowd_cache.set_crowd(mock_crowd)
        
        self.assertEquals(0, mock_crowd.get_hits())
        
        logger.debug('Requesting a reminder mail and expecting the mock to receive the call.')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.request_usernames_reminder(test_email)
        
        self.assertEquals(1, mock_crowd.get_hits())
        
        logger.debug('Requesting a reminder mail and expecting the mock to receive the call.')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = crowd_cache.request_usernames_reminder(test_email)
        
        self.assertEquals(2, mock_crowd.get_hits())
        
class MockCrowdCache(object):
    def __init__(self, existing_groups, existing_users):
        self._existing_groups = existing_groups
        self._existing_users = existing_users
        self._hits = 0

    def get_hits(self):
        return self._hits

    def get_group(self, group_id):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested group %s', group_id)
        
        if group_id in self._existing_groups:
            logger.debug('Mock cache holder: group %s is present.', group_id)
            
            return group_id
        else:
            logger.debug('Mock cache holder: group %s does not exist.', group_id)
        
            raise helpers.ObjectNotFoundException('Group not found')

    def add_group(self, group_id, group_description):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested group %s creation', group_id)
            
        if group_id in self._existing_groups:
            raise helpers.ObjectExistanceException('The Mock has already a group with the same ID')
        else:
            self._existing_groups += [group_id]
        
            return group_id
        
    def get_child_groups_of_group(self, group_id):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested child groups of group %s', group_id)
        
        if group_id in self._existing_groups:
            logger.debug('Mock cache holder: group %s is present.', group_id)
            
            possible_children = [group_id + '_child_1', group_id + '_child_2']
            
            return [child for child in possible_children if child in self._existing_groups]
        else:
            logger.debug('Mock cache holder: group %s does not exist.', group_id)
        
            raise helpers.ObjectNotFoundException('Group not found')

    def get_parent_groups_of_group(self, group_id):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested parent groups of group %s', group_id)
        
        if group_id in self._existing_groups:
            logger.debug('Mock cache holder: group %s is present.', group_id)
            
            possible_parents = [group_id + '_parent_1', group_id + '_parent_2']
            
            return [parent for parent in possible_parents if parent in self._existing_groups]
        else:
            logger.debug('Mock cache holder: group %s does not exist.', group_id)
        
            raise helpers.ObjectNotFoundException('Group not found')
        
    def add_direct_child_group_membership(self, parent_group_id, child_group_id):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested direct child relationship between %s and %s', parent_group_id, child_group_id)
        
        if not parent_group_id in self._existing_groups:
            logger.debug('Mock cache holder: parent group %s does not exist.', parent_group_id)
        
            raise helpers.ObjectNotFoundException('Parent group not found')
        elif not child_group_id in self._existing_groups:
            logger.debug('Mock cache holder: child group %s does not exist.', child_group_id)
        
            raise helpers.ObjectExistanceException('Child group not found')
        else:
            logger.debug('Mock cache holder: parent and child groups %s and %s are present.', parent_group_id, child_group_id)
            
            return True

    def get_user(self, user_id):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested user %s', user_id)
        
        if user_id in self._existing_users:
            logger.debug('Mock cache holder: user %s is present.', user_id)
            
            if isinstance(self._existing_users, dict):
                return self._existing_users[user_id]
            else:
                return user_id
        else:
            logger.debug('Mock cache holder: user %s does not exist.', user_id)
        
            raise helpers.ObjectNotFoundException('User not found')
        
    def add_user(self, user_id, email, first_name, last_name, display_name, password):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested user %s creation', user_id)
            
        if user_id in self._existing_users:
            raise helpers.ObjectExistanceException('The Mock has already a user with the same ID')
        else:
            self._existing_users += [user_id]
        
            return user_id
        
    def store_user_attributes(self, user_id, attributes_dict):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested setting attributes to user %s', user_id)
        
        if user_id in self._existing_users:
            logger.debug('Mock cache holder: user %s is present.', user_id)
            
            return True
        else:
            logger.debug('Mock cache holder: user %s does not exist.', user_id)
        
            raise helpers.ObjectNotFoundException('User not found')
        
    def add_user_to_group(self, user_id, group_id):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested adding user %s to group %s', user_id, group_id)
        
        if user_id in self._existing_users:
            if group_id in self._existing_groups:
                logger.debug('Mock cache holder: user %s and group %s are present.', user_id, group_id)
                
                return True
            else:
                logger.debug('Mock cache holder: group %s does not exist.', group_id)
            
                raise helpers.ObjectExistanceException('The group cannot be found')
        else:
            logger.debug('Mock cache holder: user %s does not exist.', user_id)
        
            raise helpers.ObjectNotFoundException('User not found')
        
    def request_usernames_reminder(self, email):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested usernames reminder for email %s', email)
        
        if email in self._existing_users:
            logger.debug('Mock cache holder: email %s is present.', email)
            
            return True
        else:
            logger.debug('Mock cache holder: email(user) %s does not exist.', email)
        
            raise helpers.ObjectNotFoundException('Email not found')

class TestConfluenceCache(unittest.TestCase):
    def test_get_from_cache_holder(self):
        holder = helpers.CacheHolder()
        
        logger.debug('Expecting that the object from the CacheHolder is the same as a new one')
        
        mock_confluence = MockConfluenceCache([''])
        confluence_cache = helpers.ConfluenceCache()
        confluence_cache.set_confluence(mock_confluence)
        
        self.assertEquals(holder.get_confluence_cache(), confluence_cache)
        self.assertEquals(holder.get_confluence_cache()._confluence, confluence_cache._confluence)
        
    def test_get_space(self):
        test_space_name = 'TESTspace'
        
        logger.debug('Initializing a confluence cache with a mock as confluence instance')
        mock_confluence = MockConfluenceCache([test_space_name])
        confluence_cache = helpers.ConfluenceCache()
        confluence_cache.set_confluence(mock_confluence)
        
        self.assertEquals(0, mock_confluence.get_hits())
        
        logger.debug('Getting a test space and expecting the mock to receive the call')
        result = confluence_cache.get_space(test_space_name)
        
        self.assertEquals(test_space_name, result)
        self.assertEquals(1, mock_confluence.get_hits())
        
        logger.debug('Getting the same test space and expecting the result to be in cache')
        result = confluence_cache.get_space(test_space_name)
        
        self.assertEquals(test_space_name, result)
        self.assertEquals(1, mock_confluence.get_hits())
        
    def test_get_space_not_existing(self):
        test_space_name = 'nonSPACE'
        
        logger.debug('Initializing a confluence cache with a mock as confluence instance, no spaces')
        mock_confluence = MockConfluenceCache([])
        confluence_cache = helpers.ConfluenceCache()
        confluence_cache.set_confluence(mock_confluence)
        
        self.assertEquals(0, mock_confluence.get_hits())
        
        logger.debug('Getting a nonexisting space and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = confluence_cache.get_space(test_space_name)
        
        self.assertEquals(1, mock_confluence.get_hits())
        
        logger.debug('Getting a nonexisting space and expecting the request not to be cached')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = confluence_cache.get_space(test_space_name)
        
        self.assertEquals(2, mock_confluence.get_hits())

    def test_get_space_permissions(self):
        test_space_name = 'TESTspace'
        
        logger.debug('Initializing a confluence cache with a mock as confluence instance')
        mock_confluence = MockConfluenceCache([test_space_name])
        confluence_cache = helpers.ConfluenceCache()
        confluence_cache.set_confluence(mock_confluence)
        
        self.assertEquals(0, mock_confluence.get_hits())
        
        logger.debug('Getting a test space permissions and expecting the mock to receive the call')
        result = confluence_cache.get_space_permissions(test_space_name)
        
        self.assertEquals(test_space_name, result)
        self.assertEquals(1, mock_confluence.get_hits())
        
        logger.debug('Getting the same test space permissions and expecting the result to be in cache')
        result = confluence_cache.get_space_permissions(test_space_name)
        
        self.assertEquals(test_space_name, result)
        self.assertEquals(1, mock_confluence.get_hits())
        
    def test_get_space_permissions_not_existing(self):
        test_space_name = 'nonSPACE'
        
        logger.debug('Initializing a confluence cache with a mock as confluence instance, no spaces')
        mock_confluence = MockConfluenceCache([])
        confluence_cache = helpers.ConfluenceCache()
        confluence_cache.set_confluence(mock_confluence)
        
        self.assertEquals(0, mock_confluence.get_hits())
        
        logger.debug('Getting a nonexisting space permissions and expecting the mock to receive the call')
        with self.assertRaises(helpers.OldAPIException):
            result = confluence_cache.get_space_permissions(test_space_name)
        
        self.assertEquals(1, mock_confluence.get_hits())
        
        logger.debug('Getting a nonexisting space permissions and expecting the request not to be cached')
        with self.assertRaises(helpers.OldAPIException):
            result = confluence_cache.get_space_permissions(test_space_name)
        
        self.assertEquals(2, mock_confluence.get_hits())
        
    def test_create_space(self):
        test_space_name = 'SPACE'
        
        logger.debug('Initializing a confluence cache with a mock as confluence instance, no spaces')
        mock_confluence = MockConfluenceCache([])
        confluence_cache = helpers.ConfluenceCache()
        confluence_cache.set_confluence(mock_confluence)
        
        self.assertEquals(0, mock_confluence.get_hits())
        
        logger.debug('Adding a space and expecting the mock to receive the call')
        result = confluence_cache.create_space(test_space_name, 'Space name', 'Test description')
        
        self.assertEquals(test_space_name, result)
        self.assertEquals(1, mock_confluence.get_hits())
        
        logger.debug('Getting the same space and expecting the request to be cached')
        result = confluence_cache.get_space(test_space_name)
        
        self.assertEquals(test_space_name, result)
        self.assertEquals(1, mock_confluence.get_hits())
        
    def test_create_space_existing(self):
        test_space_name = 'SPACEexists'
        
        logger.debug('Initializing a confluence cache with a mock as confluence instance')
        mock_confluence = MockConfluenceCache([test_space_name])
        confluence_cache = helpers.ConfluenceCache()
        confluence_cache.set_confluence(mock_confluence)
        
        self.assertEquals(0, mock_confluence.get_hits())
        
        logger.debug('Adding a space and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectExistanceException):
            result = confluence_cache.create_space(test_space_name, 'Space name', 'Test description')
        
        self.assertEquals(1, mock_confluence.get_hits())
        
        logger.debug('Getting the same space and expecting the mock to receive the call')
        result = confluence_cache.get_space(test_space_name)
        
        self.assertEquals(test_space_name, result)
        self.assertEquals(2, mock_confluence.get_hits())

    def test_add_space_permissions(self):
        test_space_name = 'SPACE'
        
        logger.debug('Initializing a confluence cache with a mock as confluence instance')
        mock_confluence = MockConfluenceCache([test_space_name])
        confluence_cache = helpers.ConfluenceCache()
        confluence_cache.set_confluence(mock_confluence)
        
        addition = helpers.SpacePermissionAddition('target')
        addition.add_to_permission_vector('NAME')
        
        self.assertEquals(0, mock_confluence.get_hits())
        
        logger.debug('Adding permissions to a space and expecting the mock to receive the call')
        result = confluence_cache.add_space_permissions(test_space_name, addition)
        
        self.assertEquals(True, result)
        self.assertEquals(1, mock_confluence.get_hits())
        
        logger.debug('Adding more permissions and expecting the mock to receive the call')
        result = confluence_cache.add_space_permissions(test_space_name, addition)
        
        self.assertEquals(True, result)
        self.assertEquals(2, mock_confluence.get_hits())

    def test_add_space_permissions_nonexisting(self):
        test_space_name = 'SPACEnonex'
        
        logger.debug('Initializing a confluence cache with a mock as confluence instance, no groups')
        mock_confluence = MockConfluenceCache([])
        confluence_cache = helpers.ConfluenceCache()
        confluence_cache.set_confluence(mock_confluence)
        
        addition = helpers.SpacePermissionAddition('target')
        addition.add_to_permission_vector('NAME')
        
        self.assertEquals(0, mock_confluence.get_hits())
        
        logger.debug('Adding permissions to a space and expecting the mock to receive the call')
        with self.assertRaises(helpers.OldAPIException):
            result = confluence_cache.add_space_permissions(test_space_name, addition)
        
        self.assertEquals(1, mock_confluence.get_hits())
        
        logger.debug('Adding more permissions and expecting the mock to receive the call')
        with self.assertRaises(helpers.OldAPIException):
            result = confluence_cache.add_space_permissions(test_space_name, addition)
            
        self.assertEquals(2, mock_confluence.get_hits())
        
    def test_get_content(self):
        test_space_name = 'SPACE'
        
        logger.debug('Initializing a confluence cache with a mock as confluence instance')
        mock_confluence = MockConfluenceCache([test_space_name])
        confluence_cache = helpers.ConfluenceCache()
        confluence_cache.set_confluence(mock_confluence)
        
        self.assertEquals(0, mock_confluence.get_hits())
        
        logger.debug('Getting the content and expecting the mock to receive the call')
        result = confluence_cache.get_content(test_space_name, 'Content Name')
        
        self.assertEquals('Content Name', result)
        self.assertEquals(1, mock_confluence.get_hits())
        
        logger.debug('Getting the same content and expecting the request not to be cached')
        result = confluence_cache.get_content(test_space_name, 'Content Name')
        
        self.assertEquals('Content Name', result)
        self.assertEquals(2, mock_confluence.get_hits())
        
    def test_get_content_nospace(self):
        test_space_name = 'noSPACE'
        
        logger.debug('Initializing a confluence cache with a mock as confluence instance')
        mock_confluence = MockConfluenceCache([])
        confluence_cache = helpers.ConfluenceCache()
        confluence_cache.set_confluence(mock_confluence)
        
        self.assertEquals(0, mock_confluence.get_hits())
        
        logger.debug('Getting the content and expecting the mock to receive the call')
        
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = confluence_cache.get_content(test_space_name, 'Content name')
        
        self.assertEquals(1, mock_confluence.get_hits())
        
        logger.debug('Getting the same content and expecting the request not to be cached')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = confluence_cache.get_content(test_space_name, 'Content name')
        
        self.assertEquals(2, mock_confluence.get_hits())
        
class MockConfluenceCache(object):
    def __init__(self, existing_spaces):
        self._existing_spaces = existing_spaces
        self._hits = 0

    def get_hits(self):
        return self._hits

    def get_space(self, space_key):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested space %s', space_key)
        
        if space_key in self._existing_spaces:
            logger.debug('Mock cache holder: space %s is present.', space_key)
            
            return space_key
        else:
            logger.debug('Mock cache holder: space %s does not exist.', space_key)
        
            raise helpers.ObjectNotFoundException('Space not found')

    def get_space_permissions(self, space_key):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested space permissions for space %s', space_key)
        
        if space_key in self._existing_spaces:
            logger.debug('Mock cache holder: space %s is present.', space_key)
            
            return space_key
        else:
            logger.debug('Mock cache holder: space %s does not exist.', space_key)
        
            raise helpers.OldAPIException('Space not found')
            
    def create_space(self, space_key, name, description):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: requested space %s creation', space_key)
            
        if space_key in self._existing_spaces:
            raise helpers.ObjectExistanceException('The Mock has already a space with the same ID')
        else:
            self._existing_spaces += [space_key]
        
            return space_key
            
    def add_space_permissions(self, space_key, addition):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: adding space permissions %s for space %s', addition, space_key)
        
        if space_key in self._existing_spaces:
            return True
        else:
            logger.debug('Mock cache holder: space %s does not exist.', space_key)
        
            raise helpers.OldAPIException('Space not found')
            
    def get_content(self, space_key, content_title):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: getting content %s in space %s', content_title , space_key)
        
        if space_key in self._existing_spaces:
            return content_title
        else:
            logger.debug('Mock cache holder: space %s does not exist.', space_key)
        
            raise helpers.ObjectNotFoundException('Space not found')
            
class TestJiraCache(unittest.TestCase):
    def test_get_from_cache_holder(self):
        holder = helpers.CacheHolder()
        
        logger.debug('Expecting that the object from the CacheHolder is the same as a new one')
        
        mock_jira = MockJiraCache([])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(holder.get_jira_cache(), jira_cache)
        self.assertEquals(holder.get_jira_cache()._jira, jira_cache._jira)

    def test_do_transition(self):
        test_issue = 'test.issue'
        test_transition = 'test.transition'
        
        logger.debug('Initializing a Jira cache with a mock as jira instance, one issue, one transition')
        mock_jira = MockJiraCache([test_issue],[test_transition])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(0, mock_jira.get_hits())
        
        logger.debug('Transitioning and expecting the mock to receive the call')
        result = jira_cache.do_transition(test_issue, test_transition)
        
        self.assertEquals(True, result)
        self.assertEquals(1, mock_jira.get_hits())
        
        logger.debug('Transitioning and expecting the mock to receive the call')
        result = jira_cache.do_transition(test_issue, test_transition)
        
        self.assertEquals(True, result)
        self.assertEquals(2, mock_jira.get_hits())
        
    def test_do_transition_invalid_issue(self):
        test_issue = 'invalid.issue'
        test_transition = 'test.transition'
        
        logger.debug('Initializing a Jira cache with a mock as jira instance, no issue, one transition')
        mock_jira = MockJiraCache([],[test_transition])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(0, mock_jira.get_hits())
        
        logger.debug('Transitioning and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = jira_cache.do_transition(test_issue, test_transition)
        
        self.assertEquals(1, mock_jira.get_hits())
        
        logger.debug('Transitioning again and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = jira_cache.do_transition(test_issue, test_transition)
        
        self.assertEquals(2, mock_jira.get_hits())
        
    def test_do_transition_issue_invalid_transition(self):
        test_issue = 'test.issue'
        test_transition = 'invalid.transition'
        
        logger.debug('Initializing a Jira cache with a mock as jira instance, one issue, no transition')
        mock_jira = MockJiraCache([test_issue],[])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(0, mock_jira.get_hits())
        
        logger.debug('Transitioning and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectExistanceException):
            result = jira_cache.do_transition(test_issue, test_transition)
        
        self.assertEquals(1, mock_jira.get_hits())
        
        logger.debug('Transitioning again and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectExistanceException):
            result = jira_cache.do_transition(test_issue, test_transition)
        
        self.assertEquals(2, mock_jira.get_hits())
        
    def test_do_transition_invalid_issue_invalid_transition(self):
        test_issue = 'invalid.issue'
        test_transition = 'invalid.transition'
        
        logger.debug('Initializing a Jira cache with a mock as jira instance, no issue, no transition')
        mock_jira = MockJiraCache([],[])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(0, mock_jira.get_hits())
        
        logger.debug('Transitioning and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = jira_cache.do_transition(test_issue, test_transition)
        
        self.assertEquals(1, mock_jira.get_hits())
        
        logger.debug('Transitioning again and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = jira_cache.do_transition(test_issue, test_transition)
        
        self.assertEquals(2, mock_jira.get_hits())
            
    def test_add_comment(self):
        test_issue = 'test.issue'
        
        logger.debug('Initializing a Jira cache with a mock as jira instance, one issue')
        mock_jira = MockJiraCache([test_issue])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(0, mock_jira.get_hits())
        
        logger.debug('Commenting and expecting the mock to receive the call')
        result = jira_cache.add_comment(test_issue, 'comment')
        
        self.assertEquals(True, result)
        self.assertEquals(1, mock_jira.get_hits())
        
        logger.debug('Commenting and expecting the mock to receive the call')
        result = jira_cache.add_comment(test_issue, 'comment')
        
        self.assertEquals(True, result)
        self.assertEquals(2, mock_jira.get_hits())
        
    def test_add_comment_invalid_issue(self):
        test_issue = 'invalid.issue'
        
        logger.debug('Initializing a Jira cache with a mock as jira instance, no issue')
        mock_jira = MockJiraCache([])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(0, mock_jira.get_hits())
        
        logger.debug('Commenting and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = jira_cache.add_comment(test_issue, 'comment')
        
        self.assertEquals(1, mock_jira.get_hits())
        
        logger.debug('Commenting and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = jira_cache.add_comment(test_issue, 'comment')
        
        self.assertEquals(2, mock_jira.get_hits())

    def test_search(self):
        test_issue1 = 'test.issue2'
        test_issue2 = 'test.issue3'
        
        logger.debug('Initializing a Jira cache with a mock as jira instance, two issues')
        mock_jira = MockJiraCache([test_issue1, test_issue2])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(0, mock_jira.get_hits())
        
        logger.debug('Searching and expecting the mock to receive the call')
        result = jira_cache.search('jql')
        
        self.assertEquals([test_issue1, test_issue2], result)
        self.assertEquals(1, mock_jira.get_hits())
        
        logger.debug('Searching and expecting the mock to receive the call')
        result = jira_cache.search('jql')
        
        self.assertEquals([test_issue1, test_issue2], result)
        self.assertEquals(2, mock_jira.get_hits())

    def test_update_fields(self):
        test_issue = 'test.issue'
        
        logger.debug('Initializing a Jira cache with a mock as jira instance, one issue')
        mock_jira = MockJiraCache([test_issue],[])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(0, mock_jira.get_hits())
        
        logger.debug('Updating fields and expecting the mock to receive the call')
        result = jira_cache.update_issue_fields(test_issue, {'field': 'value'})
        
        self.assertEquals(True, result)
        self.assertEquals(1, mock_jira.get_hits())
        
        logger.debug('Updating fields and expecting the mock to receive the call')
        result = jira_cache.update_issue_fields(test_issue, {'field': 'value'})
        
        self.assertEquals(True, result)
        self.assertEquals(2, mock_jira.get_hits())
        
    def test_update_fields_invalid_issue(self):
        test_issue = 'invalid.issue'
        
        logger.debug('Initializing a Jira cache with a mock as jira instance, no issue')
        mock_jira = MockJiraCache([],[])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(0, mock_jira.get_hits())
        
        logger.debug('Updating fields and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = jira_cache.update_issue_fields(test_issue, {'field': 'value'})
        
        self.assertEquals(1, mock_jira.get_hits())
        
        logger.debug('Updating fields again and expecting the mock to receive the call')
        with self.assertRaises(helpers.ObjectNotFoundException):
            result = jira_cache.update_issue_fields(test_issue, {'field': 'value'})
        
        self.assertEquals(2, mock_jira.get_hits())

    def test_customfield_add_option(self):
        logger.debug('Initializing a Jira cache with a mock as jira instance')
        mock_jira = MockJiraCache([],[])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(0, mock_jira.get_hits())
        
        logger.debug('Adding an option and expecting the mock to receive the call')
        result = jira_cache.customfield_add_option('10102', 'value')
        
        self.assertEquals(True, result)
        self.assertEquals(1, mock_jira.get_hits())
        
        logger.debug('Adding an option and expecting the mock to receive the call')
        result = jira_cache.customfield_add_option('10102', 'value')
        
        self.assertEquals(True, result)
        self.assertEquals(2, mock_jira.get_hits())

    def test_customfield_sort_options(self):
        logger.debug('Initializing a Jira cache with a mock as jira instance')
        mock_jira = MockJiraCache([],[])
        jira_cache = helpers.JiraCache()
        jira_cache.set_jira(mock_jira)
        
        self.assertEquals(0, mock_jira.get_hits())
        
        logger.debug('Sorting the options and expecting the mock to receive the call')
        result = jira_cache.customfield_sort_options('10102')
        
        self.assertEquals(True, result)
        self.assertEquals(1, mock_jira.get_hits())
        
        logger.debug('Sorting the options and expecting the mock to receive the call')
        result = jira_cache.customfield_sort_options('10102')
        
        self.assertEquals(True, result)
        self.assertEquals(2, mock_jira.get_hits())

class MockJiraCache(object):
    def __init__(self, issues, transitions = []):
        self._issues = issues
        self._transitions = transitions
        self._hits = 0

    def get_hits(self):
        return self._hits
            
    def search(self, query):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: searching %s', query)
        
        return self._issues
        
    def do_transition(self, issue_key, transition_id):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: issue %s transitions to %s', issue_key, transition_id)
        
        if issue_key in self._issues:
            if transition_id in self._transitions:
                logger.debug('Mock cache holder: issue %s and transition %s are present.', issue_key, transition_id)
                
                return True
            else:
                logger.debug('Mock cache holder: transition %s does not exist.', transition_id)
            
                raise helpers.ObjectExistanceException('The transition cannot be found')
        else:
            logger.debug('Mock cache holder: issue %s does not exist.', issue_key)
        
            raise helpers.ObjectNotFoundException('Issue not found')
        
    def add_comment(self, issue_key, comment_text):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: leave a comment on issue %s', issue_key)
        
        if issue_key in self._issues:
            logger.debug('Mock cache holder: issue %s is present.', issue_key)
                
            return True
        else:
            logger.debug('Mock cache holder: issue %s does not exist.', issue_key)
        
            raise helpers.ObjectNotFoundException('Issue not found')
            
    def update_issue_fields(self, issue_key, fields_dict):
        self._hits = self._hits + 1
        logger.debug('Mock cache holder: set fields on issue %s', issue_key)
        
        if issue_key in self._issues:
            logger.debug('Mock cache holder: issue %s is present.', issue_key)
                
            return True
        else:
            logger.debug('Mock cache holder: issue %s does not exist.', issue_key)
        
            raise helpers.ObjectNotFoundException('Issue not found')
            
    def customfield_add_option(self, customfield_id, option):
        self._hits = self._hits + 1
        
        return True
    
    def customfield_sort_options(self, customfield_id):
        self._hits = self._hits + 1
        
        return True
        
class MockCacheHolder(object):
    def __init__(self, crowd_groups = [], confluence_spaces = [], crowd_users = []):
        self._crowd_cache = MockCrowdCache(crowd_groups, crowd_users)
        self._confluence_cache = MockConfluenceCache(confluence_spaces)
        
    def get_crowd_cache(self):
        return self._crowd_cache
        
    def get_confluence_cache(self):
        return self._confluence_cache

if __name__ == '__main__':
    unittest.main()
