#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .context import corporatesnorlax
from corporatesnorlax import helpers
from .mockjira import MockJira
from . import mock

import unittest, logging, os

logger = logging.getLogger(__name__)

import requests, requests_mock

class TestWithMock(mock.BaseTestMock):
    def __init__(self, *args, **kwargs):
        mock.BaseTestMock.__init__(self, *args, **kwargs)
        
        self._credentials = self._all_credentials.get_jira_credentials()
        
        self._mock_jira_location = helpers.LocationHolder().get_confluence_location()
        self._mock_jira = MockJira(self._mock_jira_location)

class TestErrorCodes(TestWithMock):
    @requests_mock.Mocker()
    def test_unauthorized(self, mock):
        self._assert_error_code(mock, 'project=UNAUTHORIZED', helpers.UnauthorizedException)
        
    @requests_mock.Mocker()
    def test_forbidden(self, mock):
        self._assert_error_code(mock, 'project=FORBIDDEN', helpers.ForbiddenException)
        
    def _assert_error_code(self, mock, query, expected):
        self._mock_jira.register_uris(mock)
    
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
    
        with self.assertRaises(expected):
            found = jira.search(query)

            logger.error('Got %s', found)
            
class TestIssueObjects(TestWithMock):
    @requests_mock.Mocker()
    def test_get_empty_list(self, mock):
        self._mock_jira.register_uris(mock)
        
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
        
        issue_list_object = jira.search('project=EMPTY')

        logger.debug('Got %s', issue_list_object)

        self.assertEquals(0, len(issue_list_object.issues))
        
    @requests_mock.Mocker()
    def test_get_list(self, mock):
        self._mock_jira.register_uris(mock)
        
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
        
        issue_list_object = jira.search('project=TEST')

        logger.debug('Got %s', issue_list_object)

        self.assertEquals(3, len(issue_list_object.issues))
        
        self.assertEquals('Test issue 2', issue_list_object.issues[1].summary)
        self.assertEquals('TEST-2', issue_list_object.issues[1].issue_key)
        self.assertEquals('TEST', issue_list_object.issues[1].project_key)
        self.assertEquals('To Do', issue_list_object.issues[1].status)

class TestDoTransition(TestWithMock):
    @requests_mock.Mocker()
    def test_issue_not_found(self, mock):
        self._assert_error_code(mock, 'NOTEXISTING-1', '61', helpers.ObjectNotFoundException)
        
    @requests_mock.Mocker()
    def test_transition_not_found(self, mock):
        self._assert_error_code(mock, 'TEST-1', 'NEX', helpers.ObjectExistanceException)
        
    def _assert_error_code(self, mock, issue_key, transition_id, expected):
        self._mock_jira.register_uris(mock)
    
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
    
        with self.assertRaises(expected):
            found = jira.do_transition(issue_key, transition_id)

            logger.error('Got %s', found)
        
    @requests_mock.Mocker()
    def test_transition(self, mock):
        self._mock_jira.register_uris(mock)
        
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
        
        found = jira.do_transition('TEST-1', '61')
        
        logger.info('Got %s', found)
        
        self.assertTrue(found)
        
class TestAddComment(TestWithMock):
    @requests_mock.Mocker()
    def test_issue_not_found(self, mock):
        self._assert_error_code(mock, 'NOTEXISTING-1', '61', helpers.ObjectNotFoundException)
        
    def _assert_error_code(self, mock, issue_key, transition_id, expected):
        self._mock_jira.register_uris(mock)
    
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
    
        with self.assertRaises(expected):
            found = jira.add_comment(issue_key, 'Test comment')

            logger.error('Got %s', found)
        
    @requests_mock.Mocker()
    def test_comment(self, mock):
        self._mock_jira.register_uris(mock)
        
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
        
        found = jira.add_comment('TEST-1', 'This is the comment')
        
        logger.info('Got %s', found)
        
        self.assertTrue(found)
        
class TestUpdateFields(TestWithMock):
    @requests_mock.Mocker()
    def test_issue_not_found(self, mock):
        self._assert_error_code(mock, 'NOTEXISTING-1', helpers.ObjectNotFoundException)
        
    @requests_mock.Mocker()
    def test_issue_not_allowed(self, mock):
        self._assert_error_code(mock, 'FORBIDDEN-1', helpers.ForbiddenException)
        
    @requests_mock.Mocker()
    def test_issue_not_updated(self, mock):
        self._assert_error_code(mock, 'FAILED-1', helpers.FailedUpdateException)
        
    @requests_mock.Mocker()
    def test_issue_wrong_code(self, mock):
        self._assert_error_code(mock, 'CODE-1', helpers.UnknownResponseException)
        
    def _assert_error_code(self, mock, issue_key, expected):
        self._mock_jira.register_uris(mock)
    
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
    
        with self.assertRaises(expected):
            found = jira.update_issue_fields(issue_key, {'field1':'value'})

            logger.error('Got %s', found)
        
    @requests_mock.Mocker()
    def test_update_fields(self, mock):
        self._mock_jira.register_uris(mock)
        
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
        
        found = jira.update_issue_fields('TEST-1', {'summary': 'This is the new summary'})
        
        logger.info('Got %s', found)
        
        self.assertTrue(found)
        
    @requests_mock.Mocker()
    def test_update_fields_invalid_attributes(self, mock):
        self._assert_update_fields_invalid_attributes(mock, None)
        self._assert_update_fields_invalid_attributes(mock, [])
        self._assert_update_fields_invalid_attributes(mock, 10)
        self._assert_update_fields_invalid_attributes(mock, 'Test')
        self._assert_update_fields_invalid_attributes(mock, {})
        
    def _assert_update_fields_invalid_attributes(self, mock, fields):
        self._mock_jira.register_uris(mock)
        
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
        
        with self.assertRaises(helpers.IllegalArgumentsException):
            result = jira.update_issue_fields('TEST-1', fields)

            logger.error('Got %s', result)
            
class TestCustomFields(TestWithMock):
    @requests_mock.Mocker()
    def test_customfield_add_wrong_field(self, mock):
        self._assert_customfield_add_error_code(mock, '0001', helpers.ObjectExistanceException)
        
    @requests_mock.Mocker()
    def test_customfield_sort_wrong_field(self, mock):
        self._assert_customfield_sort_error_code(mock, '0001', helpers.FailedUpdateException)

    @requests_mock.Mocker()
    def test_customfield_add_inexistent_field(self, mock):
        self._assert_customfield_add_error_code(mock, '0002', helpers.ObjectNotFoundException)
        
    @requests_mock.Mocker()
    def test_customfield_sort_inexistent_field(self, mock):
        self._assert_customfield_sort_error_code(mock, '0002', helpers.ObjectNotFoundException)

    @requests_mock.Mocker()
    def test_customfield_add_unauthorized(self, mock):
        self._assert_customfield_add_error_code(mock, '0003', helpers.UnauthorizedException)
        
    @requests_mock.Mocker()
    def test_customfield_sort_unauthorized(self, mock):
        self._assert_customfield_sort_error_code(mock, '0003', helpers.UnauthorizedException)

    @requests_mock.Mocker()
    def test_customfield_add_forbidden(self, mock):
        self._assert_customfield_add_error_code(mock, '0004', helpers.ForbiddenException)
        
    @requests_mock.Mocker()
    def test_customfield_sort_forbidden(self, mock):
        self._assert_customfield_sort_error_code(mock, '0004', helpers.ForbiddenException)

    def _assert_customfield_add_error_code(self, mock, customfield, expected):
        self._mock_jira.register_uris(mock)
    
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
    
        with self.assertRaises(expected):
            found = jira.customfield_add_option(customfield, 'option')

            logger.error('Got %s', found)
            
    def _assert_customfield_sort_error_code(self, mock, customfield, expected):
        self._mock_jira.register_uris(mock)
    
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
    
        with self.assertRaises(expected):
            found = jira.customfield_sort_options(customfield)
            
            logger.error('Got %s', found)

    @requests_mock.Mocker()
    def test_customfield_add(self, mock):
        self._mock_jira.register_uris(mock)
        
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
        
        found = jira.customfield_add_option('4711', 'new_option')
        
        logger.info('Got %s', found)
        
        self.assertTrue(found)
            
    @requests_mock.Mocker()
    def test_customfield_sort(self, mock):
        self._mock_jira.register_uris(mock)
        
        jira = corporatesnorlax.JiraInterface(self._credentials, self._mock_jira_location)
        
        found = jira.customfield_sort_options('4711')
        
        logger.info('Got %s', found)
        
        self.assertTrue(found)
            

