#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .context import corporatesnorlax
from corporatesnorlax import helpers
from .mockcrowd import MockCrowd

import unittest, logging

logger = logging.getLogger(__name__)

import requests, requests_mock

class TestRequestsMock(unittest.TestCase):
    """Basic test cases."""
    
    def _init_mock(self, mock):
        logger.info('Registering the URIs')
        mock.register_uri(url='/', method='GET', request_headers={'key': 'val'}, text='X')
        mock.register_uri(url='/tst', method='GET', text='Y')
        mock.add_matcher(self.get_post_matcher('/past', {'A':'10'}, 'DATA1'))
        mock.add_matcher(self.get_post_matcher('/past', {'B':'30'}, 'DATA2'))

    def get_post_matcher(self, url, json, content = ''):
        def post_matcher(request):
            logger.info('Got POST data %s', request.text)
            if request.method == 'POST':
                if request.path_url == url and request.text == content:
                    resp = requests.Response()
                    resp.json = json
                    resp.status_code = 200
                    return resp
                else:
                    logger.error('Not matched %s=%s, %s=%s', request.path_url, url, request.text, content)
                    return None
            return None
        return post_matcher
        
    def test_absolute_truth_and_meaning(self):
        assert True

    @requests_mock.Mocker()
    def test_requests(self, mock):
        self._init_mock(mock)
        
        result = requests.get('http://localhost/', headers={'key': 'val'})
        logger.debug('Got result %s', result)
        self.assertEquals('X', result.text)
        
        result = requests.get('https://www.google.com/', headers={'key': 'val'})
        logger.debug('Got result %s', result)
        self.assertEquals('X', result.text)
        
        result = requests.get('http://localhost/tst')
        logger.debug('Got result %s', result)
        self.assertEquals('Y', result.text)
        
        result = requests.get('http://por.le/tst')
        logger.debug('Got result %s', result)
        self.assertEquals('Y', result.text)

    @requests_mock.Mocker()
    def test_post_requests(self, mock):
        self._init_mock(mock)

        result = requests.post('http://por.le/past', data='DATA1')
        logger.debug('Got result %s', result)
        self.assertEquals({'A':'10'}, result.json)

        result = requests.post('http://por.le/past', data='DATA2')
        logger.debug('Got result %s', result)
        self.assertEquals({'B':'30'}, result.json)

crowd_url = "https://crowdserver/crowd"
crowd_location = helpers.Location()
crowd_location._url = crowd_url 
crowd = MockCrowd(crowd_location)

def get_user(username):
    url = crowd_url + '/rest/usermanagement/1/user?username=' + username + '&expand=attributes'
    logger.info("Sending GET request to %s", url)
    
    return requests.get(url, headers={'Accept': 'application/json', 'Authorization': 'Basic Y2hvc2VuLXVzZXJuYW1lOmh1bnRlcjI='})

class TestMockCrowdWorkings(unittest.TestCase):
    @requests_mock.Mocker()
    def test_get_user(self, mock):
        crowd.register_uris(mock)
        
        response = get_user('existing.user')
        
        logger.debug("Got response: %s", response)
        self.assertEquals(200, response.status_code)
        self.assertEquals('sample@user.cool', response.json()['email'])

    @requests_mock.Mocker()
    def test_get_user_invalid(self, mock):
        crowd.register_uris(mock)
        
        response = get_user('invalid.user')
        
        logger.debug("Got response: %s", response)
        self.assertEquals(404, response.status_code)

if __name__ == '__main__':
    unittest.main()
