#!/usr/bin/python3
# 
# Copyright (c) Andrea Micheloni 2017
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .context import corporatesnorlax
from corporatesnorlax import helpers
from corporatesnorlax import usecases
from .test_helpers import MockCacheHolder

import unittest, logging

logger = logging.getLogger(__name__)

class TestStructureAction(unittest.TestCase):
    def test_create_group_action(self):
        action = usecases.CreateGroupAction('test.group', 'Test Group')
        
        self.assertEquals('CreateGroupAction(\'test.group\', \'Test Group\')', str(action))

    def test_create_group_action_requirements(self):
        action = usecases.CreateGroupAction('test.group', 'Test Group')
        
        self.assertEquals('[GroupExistanceRequirement(\'test.group\', False)]', str(action.get_requirements()))

    def test_create_user_action(self):
        action = usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password')
        
        self.assertEquals('CreateUserAction(\'test.user\', \'test@mail.tld\', \'Test\', \'User\', \'Test User\', \'***\')', str(action))

    def test_create_user_action_requirements(self):
        action = usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password')
        
        self.assertEquals('[UserExistanceRequirement(\'test.user\', False)]', str(action.get_requirements()))

    def test_add_user_to_group_action(self):
        action = usecases.AddUserToGroupAction('test.user', 'test.group')
        
        self.assertEquals('AddUserToGroupAction(\'test.user\', \'test.group\')', str(action))

    def test_add_user_to_group_action_requirements(self):
        action = usecases.AddUserToGroupAction('test.user', 'test.group')
        
        self.assertEquals('[UserExistanceRequirement(\'test.user\', True), GroupExistanceRequirement(\'test.group\', True)]', str(action.get_requirements()))

    def test_link_groups_action(self):
        action = usecases.LinkGroupsAction('test.group1', 'test.group2')
        
        self.assertEquals('LinkGroupsAction(\'test.group1\', \'test.group2\')', str(action))

    def test_link_groups_action_requirements(self):
        action = usecases.LinkGroupsAction('test.group1', 'test.group2')
        
        self.assertEquals('[GroupExistanceRequirement(\'test.group1\', True), GroupExistanceRequirement(\'test.group2\', True)]', str(action.get_requirements()))

    def test_set_user_attributes_action(self):
        action = usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'})
        
        self.assertEquals('SetUserAttributesAction(\'test.user\', {\'attribute1\': \'value1\'})', str(action))

    def test_set_user_attributes_action_requirements(self):
        action = usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'})
        
        self.assertEquals('[UserExistanceRequirement(\'test.user\', True)]', str(action.get_requirements()))
        
    def test_send_user_reminder_action(self):
        action = usecases.SendUsernameReminderAction('test.user')
        
        self.assertEquals('SendUsernameReminderAction(\'test.user\')', str(action))

    def test_send_user_reminder_action_requirements(self):
        action = usecases.SendUsernameReminderAction('test.user')
        
        self.assertEquals('[UserExistanceRequirement(\'test.user\', True)]', str(action.get_requirements()))

    def test_run_create_group(self):
        action = usecases.CreateGroupAction('test.group', 'Test Group')
        cache = MockCacheHolder([])
                
        self.assertTrue(action.run(cache))

    def test_run_create_group_existing(self):
        action = usecases.CreateGroupAction('existing.group', 'Existing Group')
        cache = MockCacheHolder(['existing.group'])
                
        self.assertFalse(action.run(cache))

    def test_run_create_user(self):
        action = usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password')
        cache = MockCacheHolder([], [], [])
                
        self.assertTrue(action.run(cache))

    def test_run_create_user_existing(self):
        action = usecases.CreateUserAction('existing.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password')
        cache = MockCacheHolder([], [], ['existing.user'])
                
        self.assertFalse(action.run(cache))

    def test_run_add_user_to_group(self):
        action = usecases.AddUserToGroupAction('test.user', 'test.group')
        cache = MockCacheHolder(['test.group'], [], ['test.user'])
                
        self.assertTrue(action.run(cache))

    def test_run_add_user_to_group_invalid_user(self):
        action = usecases.AddUserToGroupAction('invalid.user', 'test.group')
        cache = MockCacheHolder(['test.group'], [], [])
                
        self.assertFalse(action.run(cache))
        
    def test_run_add_user_to_group_invalid_group(self):
        action = usecases.AddUserToGroupAction('test.user', 'invalid.group')
        cache = MockCacheHolder([], [], ['test.user'])
                
        self.assertFalse(action.run(cache))
        
    def test_run_add_user_to_group_invalid_both(self):
        action = usecases.AddUserToGroupAction('invalid.user', 'invalid.group')
        cache = MockCacheHolder([], [], [])
                
        self.assertFalse(action.run(cache))

    def test_run_link_groups(self):
        action = usecases.LinkGroupsAction('parent.group', 'child.group')
        cache = MockCacheHolder(['parent.group', 'child.group'])
                
        self.assertTrue(action.run(cache))

    def test_run_link_groups_invalid_parent(self):
        action = usecases.LinkGroupsAction('invalid.parent.group', 'child.group')
        cache = MockCacheHolder(['child.group'])
                
        self.assertFalse(action.run(cache))

    def test_run_link_groups_invalid_child(self):
        action = usecases.LinkGroupsAction('parent.group', 'invalid.child.group')
        cache = MockCacheHolder(['parent.group'])
                
        self.assertFalse(action.run(cache))
        
    def test_run_link_groups_invalid_groups(self):
        action = usecases.LinkGroupsAction('invalid.parent.group', 'invalid.child.group')
        cache = MockCacheHolder([])
                
        self.assertFalse(action.run(cache))

    def test_run_set_user_attributes(self):
        action = usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'})
        cache = MockCacheHolder([], [], ['test.user'])
                
        self.assertTrue(action.run(cache))

    def test_run_set_user_attributes_invalid(self):
        action = usecases.SetUserAttributesAction('invalid.user', {'attribute1': 'value1'})
        cache = MockCacheHolder([], [], [])
        
        self.assertFalse(action.run(cache))
        
    def test_run_send_user_reminder(self):
        action = usecases.SendUsernameReminderAction('test.user')
        user = helpers.UserObject()
        user.email='test@mail.tld'
        
        cache = MockCacheHolder([], [], {'test.user': user, user.email: None})
                
        self.assertTrue(action.run(cache))

    def test_run_send_user_reminder_nomail(self):
        action = usecases.SendUsernameReminderAction('test.user')
        user = helpers.UserObject()
        user.email=None
        
        cache = MockCacheHolder([], [], {'test.user': user})
                
        self.assertFalse(action.run(cache))

    def test_run_send_user_reminder_invalid(self):
        action = usecases.SendUsernameReminderAction('invalid.user')
        cache = MockCacheHolder([], [], [])
        
        self.assertFalse(action.run(cache))

    def test_create_space_action(self):
        action = usecases.CreateSpaceAction('TEST', 'Test Space', 'Description of the test space')
        
        self.assertEquals('CreateSpaceAction(\'TEST\', \'Test Space\', \'Description of the test space\')', str(action))

    def test_create_space_action_requirements(self):
        action = usecases.CreateSpaceAction('TEST', 'Test Space', 'Description of the test space')
        
        self.assertEquals('[SpaceExistanceRequirement(\'TEST\', False)]', str(action.get_requirements()))

    def test_add_space_permissions_action(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        action = usecases.AddSpacePermissionsAction('SPACE', addition)
        
        self.assertEquals('AddSpacePermissionsAction(\'SPACE\', <SpacePermissionAddition target=\'target_group\', target_is_group=True, permission_vector=[\'PER_1\', \'PER_2\'] >)', str(action))

    def test_add_space_permissions_requirements(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        action = usecases.AddSpacePermissionsAction('SPACE', addition)
        
        self.assertEquals('[SpaceExistanceRequirement(\'SPACE\', True), GroupExistanceRequirement(\'target_group\', True)]', str(action.get_requirements()))

    def test_run_create_space(self):
        action = usecases.CreateSpaceAction('TEST', 'Test Space', 'Description of the test space')
        cache = MockCacheHolder(None, [])
                
        self.assertTrue(action.run(cache))

    def test_run_create_space_existing(self):
        action = usecases.CreateSpaceAction('TEST', 'Test Space', 'Description of the test space')
        cache = MockCacheHolder(None, ['TEST'])
                
        self.assertFalse(action.run(cache))

    def test_run_add_space_permissions(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        action = usecases.AddSpacePermissionsAction('SPACE', addition)
        
        cache = MockCacheHolder(['target_group'], ['SPACE'])
                
        self.assertTrue(action.run(cache))

    def test_run_add_space_permissions_nospace(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        action = usecases.AddSpacePermissionsAction('SPACE', addition)
        
        cache = MockCacheHolder(['target_group'], [])
                
        self.assertFalse(action.run(cache))

    def test_run_add_space_permissions_nogroup(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        action = usecases.AddSpacePermissionsAction('SPACE', addition)
        
        cache = MockCacheHolder(None, ['SPACE'])
                
        logger.warning('Expecting True, because the MockConfluence does not know about the non-existing group in MockCrowd')
        self.assertTrue(action.run(cache))

class TestStructureUsecase(unittest.TestCase):
    def test_requirements_group_creation(self):
        actions = [usecases.CreateGroupAction('test.group1', 'Test Group 1')]

        self._assert_requirements('[GroupExistanceRequirement(\'test.group1\', False)]', actions)

    def test_requirements_user_creation(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password')]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', False)]', actions)



    def test_requirements_set_user_attributes(self):
        actions = [usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'})]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', True)]', actions)
        
    def test_requirements_send_user_reminder(self):
        actions = [usecases.SendUsernameReminderAction('test.user')]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', True)]', actions)

    def test_requirements_set_attributes_send_user_reminder(self):
        actions = [usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'}),
                    usecases.SendUsernameReminderAction('test.user')]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', True)]', actions)

    def test_requirements_create_user_set_user_attributes(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'})]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', False)]', actions)
        
    def test_requirements_create_user_send_user_reminder(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.SendUsernameReminderAction('test.user')]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', False)]', actions)
        
    def test_requirements_create_user_set_attributes_send_user_reminder(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'}),
                    usecases.SendUsernameReminderAction('test.user')]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', False)]', actions)

    def test_requirements_add_user_to_group(self):
        actions = [usecases.AddUserToGroupAction('test.user', 'test.group')]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', True), GroupExistanceRequirement(\'test.group\', True)]', actions)
        
    def test_requirements_create_user_add_user_to_group(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.AddUserToGroupAction('test.user', 'test.group')]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', False), GroupExistanceRequirement(\'test.group\', True)]', actions)
        
    def test_requirements_create_user_add_user_to_groups(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.AddUserToGroupAction('test.user', 'test.group1'),
                    usecases.AddUserToGroupAction('test.user', 'test.group2'),
                    usecases.AddUserToGroupAction('test.user', 'test.group3')]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', False), GroupExistanceRequirement(\'test.group1\', True), GroupExistanceRequirement(\'test.group2\', True), GroupExistanceRequirement(\'test.group3\', True)]', actions)
        
    def test_requirements_create_user_add_user_to_groups_set_user_attributes(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.AddUserToGroupAction('test.user', 'test.group1'),
                    usecases.AddUserToGroupAction('test.user', 'test.group2'),
                    usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'})]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', False), GroupExistanceRequirement(\'test.group1\', True), GroupExistanceRequirement(\'test.group2\', True)]', actions)
        
    def test_requirements_create_user_add_user_to_groups_set_user_attributes_send_user_reminder(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.AddUserToGroupAction('test.user', 'test.group1'),
                    usecases.AddUserToGroupAction('test.user', 'test.group2'),
                    usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'}),
                    usecases.SendUsernameReminderAction('test.user')]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', False), GroupExistanceRequirement(\'test.group1\', True), GroupExistanceRequirement(\'test.group2\', True)]', actions)
        
    def test_requirements_create_group_add_user_to_group(self):
        actions = [usecases.CreateGroupAction('test.group', 'Test Group 1'),
                    usecases.AddUserToGroupAction('test.user', 'test.group')]

        self._assert_requirements('[GroupExistanceRequirement(\'test.group\', False), UserExistanceRequirement(\'test.user\', True)]', actions)
        
    def test_requirements_create_user_create_group_add_user_to_group(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.CreateGroupAction('test.group', 'Test Group 1'),
                    usecases.AddUserToGroupAction('test.user', 'test.group')]

        self._assert_requirements('[UserExistanceRequirement(\'test.user\', False), GroupExistanceRequirement(\'test.group\', False)]', actions)

    def test_requirements_space_creation(self):
        actions = [usecases.CreateSpaceAction('TEST', 'Test Space', 'Description of the test space')]

        self._assert_requirements('[SpaceExistanceRequirement(\'TEST\', False)]', actions)

    def test_requirements_double_space_creation(self):
        actions = [usecases.CreateSpaceAction('TEST', 'Test Space', 'Description of the test space'),
                    usecases.CreateSpaceAction('TESTinternal', 'Test Space (internal)', 'Description of the internal test space')]

        self._assert_requirements('[SpaceExistanceRequirement(\'TEST\', False), SpaceExistanceRequirement(\'TESTinternal\', False)]', actions)

    def test_requirements_add_space_permissions(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.AddSpacePermissionsAction('SPACE', addition)]

        self._assert_requirements('[SpaceExistanceRequirement(\'SPACE\', True), GroupExistanceRequirement(\'target_group\', True)]', actions)

    def test_requirements_add_space_permissions_after_group_creation(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.CreateGroupAction('target_group', 'Target Group'), usecases.AddSpacePermissionsAction('SPACE', addition)]

        self._assert_requirements('[GroupExistanceRequirement(\'target_group\', False), SpaceExistanceRequirement(\'SPACE\', True)]', actions)

    def test_requirements_add_space_permissions_after_group_and_space_creation(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.CreateGroupAction('target_group', 'Target Group'), usecases.CreateSpaceAction('SPACE', 'Test Space', 'Description of the test space'), usecases.AddSpacePermissionsAction('SPACE', addition)]

        self._assert_requirements('[GroupExistanceRequirement(\'target_group\', False), SpaceExistanceRequirement(\'SPACE\', False)]', actions)

    def test_check_requirements_group_creation(self):
        actions = [usecases.CreateGroupAction('test.group1', 'Test Group 1')]
        groups = []
    
        self._assert_check_requirements(True, actions, groups, [], [])

    def test_check_requirements_group_creation_exists(self):
        actions = [usecases.CreateGroupAction('test.group1', 'Test Group 1')]
        groups = ['test.group1']
    
        self._assert_check_requirements(False, actions, groups, [], [])

    def test_check_requirements_user_creation(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password')]
        users = []
    
        self._assert_check_requirements(True, actions, [], [], users)

    def test_check_requirements_user_creation_exists(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password')]
        users = ['test.user']
    
        self._assert_check_requirements(False, actions, [], [], users)

    def test_check_requirements_set_attributes_send_user_reminder(self):
        actions = [usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'}),
                    usecases.SendUsernameReminderAction('test.user')]
        
        users = ['test.user']
        
        self._assert_check_requirements(True, actions, [], [], users)

    def test_check_requirements_set_attributes_send_user_reminder_invalid(self):
        actions = [usecases.SetUserAttributesAction('invalid.user', {'attribute1': 'value1'}),
                    usecases.SendUsernameReminderAction('test.user')]
        
        users = []
        
        self._assert_check_requirements(False, actions, [], [], users)

    def test_check_requirements_create_user_set_user_attributes(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'})]
        
        users = []
        
        self._assert_check_requirements(True, actions, [], [], users)
        
    def test_check_requirements_create_user_set_user_attributes_existing(self):
        actions = [usecases.CreateUserAction('existing.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'})]
        
        users = ['existing.user']
        
        self._assert_check_requirements(False, actions, [], [], users)
        
    def test_check_requirements_create_user_send_user_reminder(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.SendUsernameReminderAction('test.user')]
        
        users = []
        
        self._assert_check_requirements(True, actions, [], [], users)
        
    def test_check_requirements_create_user_send_user_reminder_existing(self):
        actions = [usecases.CreateUserAction('existing.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.SendUsernameReminderAction('test.user')]
        
        users = ['existing.user']
        
        self._assert_check_requirements(False, actions, [], [], users)
        
    def test_check_requirements_create_user_set_attributes_send_user_reminder(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'}),
                    usecases.SendUsernameReminderAction('test.user')]
        
        users = []
        
        self._assert_check_requirements(True, actions, [], [], users)
        
    def test_check_requirements_create_user_set_attributes_send_user_reminder_existing(self):
        actions = [usecases.CreateUserAction('existing.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.SetUserAttributesAction('test.user', {'attribute1': 'value1'}),
                    usecases.SendUsernameReminderAction('test.user')]
        
        users = ['existing.user']
        
        self._assert_check_requirements(False, actions, [], [], users)

    def test_check_requirements_add_user_to_group(self):
        actions = [usecases.AddUserToGroupAction('test.user', 'test.group')]
        users = ['test.user']
        groups = ['test.group']
        
        self._assert_check_requirements(True, actions, groups, [], users)

    def test_check_requirements_add_user_to_group_invalid_user(self):
        actions = [usecases.AddUserToGroupAction('invalid.user', 'test.group')]
        users = []
        groups = ['test.group']
        
        self._assert_check_requirements(False, actions, groups, [], users)
        
    def test_check_requirements_add_user_to_group_invalid_group(self):
        actions = [usecases.AddUserToGroupAction('test.user', 'invalid.group')]
        users = ['test.user']
        groups = []
        
        self._assert_check_requirements(False, actions, groups, [], users)
        
    def test_check_requirements_add_user_to_group_invalid_both(self):
        actions = [usecases.AddUserToGroupAction('invalid.user', 'invalid.group')]
        users = []
        groups = []
        
        self._assert_check_requirements(False, actions, groups, [], users)
        
    def test_check_requirements_create_user_add_user_to_group(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'), 
                    usecases.AddUserToGroupAction('test.user', 'test.group')]
        users = []
        groups = ['test.group']
        
        self._assert_check_requirements(True, actions, groups, [], users)
        
    def test_check_requirements_create_existing_user_add_user_to_group(self):
        actions = [usecases.CreateUserAction('existing.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'),
                    usecases.AddUserToGroupAction('test.user', 'test.group')]
        users = ['existing.user']
        groups = ['test.group']
        
        self._assert_check_requirements(False, actions, groups, [], users)
        
    def test_check_requirements_create_user_add_user_to_invalid_group(self):
        actions = [usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'), 
                    usecases.AddUserToGroupAction('test.user', 'invalid.group')]
        users = []
        groups = []
        
        self._assert_check_requirements(False, actions, groups, [], users)
        
    def test_check_requirements_create_group_add_user_to_group(self):
        actions = [usecases.CreateGroupAction('test.group', 'Test Group 1'),
                    usecases.AddUserToGroupAction('test.user', 'test.group')]
        users = ['test.user']
        groups = []
        
        self._assert_check_requirements(True, actions, groups, [], users)
        
    def test_check_requirements_create_existing_group_add_user_to_group(self):
        actions = [usecases.CreateGroupAction('existing.group', 'Test Group 1'),
                    usecases.AddUserToGroupAction('test.user', 'existing.group')]
        users = ['test.user']
        groups = ['existing.group']
        
        self._assert_check_requirements(False, actions, groups, [], users)
        
    def test_check_requirements_create_group_add_invalid_user_to_group(self):
        actions = [usecases.CreateGroupAction('test.group', 'Test Group 1'),
                    usecases.AddUserToGroupAction('invalid.user', 'test.group')]
        users = []
        groups = []
        
        self._assert_check_requirements(False, actions, groups, [], users)
        
    def test_check_requirements_create_group_create_user_add_user_to_group(self):
        actions = [usecases.CreateGroupAction('test.group', 'Test Group 1'),
                    usecases.CreateUserAction('test.user', 'test@mail.tld', 'Test', 'User', 'Test User', 'password'), 
                    usecases.AddUserToGroupAction('test.user', 'test.group')]
        users = []
        groups = []
        
        self._assert_check_requirements(True, actions, groups, [], users)

    def test_requirements_group_linkage(self):
        actions = [usecases.LinkGroupsAction('test.group1', 'test.group2')]

        self._assert_requirements('[GroupExistanceRequirement(\'test.group1\', True), GroupExistanceRequirement(\'test.group2\', True)]', actions)
        
    def test_check_requirements_group_linkage(self):
        actions = [usecases.LinkGroupsAction('test.group1', 'test.group2')]
        groups = ['test.group1', 'test.group2']
    
        self._assert_check_requirements(True, actions, groups, [], [])

    def test_check_requirements_group_linkage_nonexisting(self):
        actions = [usecases.LinkGroupsAction('test.group1', 'test.group2')]
        groups = []
    
        self._assert_check_requirements(False, actions, groups, [], [])

    def test_check_requirements_group_linkage_partialexisting(self):
        actions = [usecases.LinkGroupsAction('test.group1', 'test.group2')]
        groups = ['test.group1']
    
        self._assert_check_requirements(False, actions, groups, [], [])

    def test_requirements_group_creation_and_linkage(self):
        actions = [usecases.CreateGroupAction('test.group1', 'Test Group 1'), usecases.LinkGroupsAction('test.group1', 'test.group2')]

        self._assert_requirements('[GroupExistanceRequirement(\'test.group1\', False), GroupExistanceRequirement(\'test.group2\', True)]', actions)

    def test_check_requirements_group_creation_and_linkage(self):
        actions = [usecases.CreateGroupAction('test.group1', 'Test Group 1'), usecases.LinkGroupsAction('test.group1', 'test.group2')]
        groups = ['test.group2']
    
        self._assert_check_requirements(True, actions, groups, [], [])

    def test_check_requirements_spaces_creation(self):
        actions = [usecases.CreateSpaceAction('TEST', 'Test Space', 'Description of the test space')]
        spaces = []
    
        self._assert_check_requirements(True, actions, [], spaces, [])

    def test_check_requirements_spaces_creation_existing(self):
        actions = [usecases.CreateSpaceAction('TEST', 'Test Space', 'Description of the test space')]
        spaces = ['TEST']
    
        self._assert_check_requirements(False, actions, [], spaces, [])

    def test_check_requirements_add_space_permissions(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = ['SPACE']
        groups = ['target_group']
        
        self._assert_check_requirements(True, actions, groups, spaces, [])

    def test_check_requirements_add_space_permissions_nogroup(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = ['SPACE']
        groups = []
        
        self._assert_check_requirements(False, actions, groups, spaces, [])

    def test_check_requirements_add_space_permissions_nospace(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = []
        groups = ['target_group']
        
        self._assert_check_requirements(False, actions, groups, spaces, [])

    def test_check_requirements_add_space_permissions_nonothing(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = []
        groups = []
        
        self._assert_check_requirements(False, actions, groups, spaces, [])

    def test_check_requirements_add_space_permissions_after_group_creation(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.CreateGroupAction('target_group', 'Target Group'), usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = ['SPACE']
        groups = []
        
        self._assert_check_requirements(True, actions, groups, spaces, [])

    def test_check_requirements_add_space_permissions_after_group_creation_exists(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.CreateGroupAction('target_group', 'Target Group'), usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = ['SPACE']
        groups = ['target_group']
        
        self._assert_check_requirements(False, actions, groups, spaces, [])

    def test_check_requirements_add_space_permissions_after_space_creation(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.CreateSpaceAction('SPACE', 'Test Space', 'Description of the test space'), usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = []
        groups = ['target_group']
        
        self._assert_check_requirements(True, actions, groups, spaces, [])

    def test_check_requirements_add_space_permissions_after_space_creation_exists(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.CreateSpaceAction('SPACE', 'Test Space', 'Description of the test space'), usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = ['SPACE']
        groups = ['target_group']
        
        self._assert_check_requirements(False, actions, groups, spaces, [])


    def test_check_requirements_add_space_permissions_after_group_and_space_creation(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.CreateGroupAction('target_group', 'Target Group'), usecases.CreateSpaceAction('SPACE', 'Test Space', 'Description of the test space'), usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = []
        groups = []
        
        self._assert_check_requirements(True, actions, groups, spaces, [])

    def test_check_requirements_add_space_permissions_after_group_and_space_creation_exist01(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.CreateGroupAction('target_group', 'Target Group'), usecases.CreateSpaceAction('SPACE', 'Test Space', 'Description of the test space'), usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = ['SPACE']
        groups = []
        
        self._assert_check_requirements(False, actions, groups, spaces, [])

    def test_check_requirements_add_space_permissions_after_group_and_space_creation_exist10(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.CreateGroupAction('target_group', 'Target Group'), usecases.CreateSpaceAction('SPACE', 'Test Space', 'Description of the test space'), usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = []
        groups = ['target_group']
        
        self._assert_check_requirements(False, actions, groups, spaces, [])

    def test_check_requirements_add_space_permissions_after_group_and_space_creation_exist11(self):
        addition = helpers.SpacePermissionAddition('target_group')
        addition.add_to_permission_vector('PER_1')
        addition.add_to_permission_vector('PER_2')
        actions = [usecases.CreateGroupAction('target_group', 'Target Group'), usecases.CreateSpaceAction('SPACE', 'Test Space', 'Description of the test space'), usecases.AddSpacePermissionsAction('SPACE', addition)]
        
        spaces = ['SPACE']
        groups = ['target_group']
        
        self._assert_check_requirements(False, actions, groups, spaces, [])

    def test_run_actions_1_ko_reqs(self):
        self._assert_run_actions(False, [False], [(False, False)], False)
        self._assert_run_actions(False, [False], [(False, True)], False)
       
    def test_run_actions_1_ok_reqs(self):
        self._assert_run_actions(False, [True], [(True, False)], False)
        self._assert_run_actions(True, [True], [(True, True)], False)
        
    def test_run_actions_1_ko_reqs_force(self):
        self._assert_run_actions(True, [True], [(False, False)], True)
        self._assert_run_actions(True, [True], [(False, True)], True)
       
    def test_run_actions_1_ok_reqs_force(self):
        self._assert_run_actions(True, [True], [(True, False)], True)
        self._assert_run_actions(True, [True], [(True, True)], True)
        
    def test_run_actions_2_ko_ko_reqs(self):
        self._assert_run_actions(False, [False, False], [(False, False), (False, False)], False)
        self._assert_run_actions(False, [False, False], [(False, False), (False, True)], False)
        self._assert_run_actions(False, [False, False], [(False, True), (False, False)], False)
        self._assert_run_actions(False, [False, False], [(False, True), (False, True)], False)

    def test_run_actions_2_ko_ok_reqs(self):
        self._assert_run_actions(False, [False, False], [(False, False), (True, False)], False)
        self._assert_run_actions(False, [False, False], [(False, False), (True, True)], False)
        self._assert_run_actions(False, [False, False], [(False, True), (True, False)], False)
        self._assert_run_actions(False, [False, False], [(False, True), (True, True)], False)
        
    def test_run_actions_2_ok_ko_reqs(self):
        self._assert_run_actions(False, [False, False], [(True, False), (False, False)], False)
        self._assert_run_actions(False, [False, False], [(True, False), (False, True)], False)
        self._assert_run_actions(False, [False, False], [(True, True), (False, False)], False)
        self._assert_run_actions(False, [False, False], [(True, True), (False, True)], False)
        
    def test_run_actions_2_ok_ok_reqs(self):
        self._assert_run_actions(False, [True, False], [(True, False), (True, False)], False)
        self._assert_run_actions(False, [True, False], [(True, False), (True, True)], False)
        self._assert_run_actions(False, [True, True], [(True, True), (True, False)], False)
        self._assert_run_actions(True, [True, True], [(True, True), (True, True)], False)

    def test_run_actions_2_ko_ko_reqs_force(self):
        self._assert_run_actions(True, [True, True], [(False, False), (False, False)], True)
        self._assert_run_actions(True, [True, True], [(False, False), (False, True)], True)
        self._assert_run_actions(True, [True, True], [(False, True), (False, False)], True)
        self._assert_run_actions(True, [True, True], [(False, True), (False, True)], True)

    def test_run_actions_2_ko_ok_reqs_force(self):
        self._assert_run_actions(True, [True, True], [(False, False), (True, False)], True)
        self._assert_run_actions(True, [True, True], [(False, False), (True, True)], True)
        self._assert_run_actions(True, [True, True], [(False, True), (True, False)], True)
        self._assert_run_actions(True, [True, True], [(False, True), (True, True)], True)
        
    def test_run_actions_2_ok_ko_reqs_force(self):
        self._assert_run_actions(True, [True, True], [(True, False), (False, False)], True)
        self._assert_run_actions(True, [True, True], [(True, False), (False, True)], True)
        self._assert_run_actions(True, [True, True], [(True, True), (False, False)], True)
        self._assert_run_actions(True, [True, True], [(True, True), (False, True)], True)
        
    def test_run_actions_2_ok_ok_reqs_force(self):
        self._assert_run_actions(True, [True, True], [(True, False), (True, False)], True)
        self._assert_run_actions(True, [True, True], [(True, False), (True, True)], True)
        self._assert_run_actions(True, [True, True], [(True, True), (True, False)], True)
        self._assert_run_actions(True, [True, True], [(True, True), (True, True)], True)
        
    def _assert_check_requirements(self, expected_satisfied, actions, groups, spaces, users):
        usecase = MockStructureUsecase(actions)
        cache = MockCacheHolder(groups, spaces, users)
        
        logger.info('Expecting that with existing groups %s and actions %s the result of the requirements is %s', groups, actions, expected_satisfied)

        result = usecase.check_all_requirements(cache)

        logger.debug('Got result: %s', result)

        self.assertEquals(expected_satisfied, result)
        
    def _assert_requirements(self, expected_representation, actions):
        usecase = MockStructureUsecase(actions)
        
        logger.info('Expecting that actions %s give requirements %s', actions, expected_representation)

        representation = str(usecase._get_all_requirements())
        
        logger.debug('Found representation: %s', representation)
        
        self.assertEquals(expected_representation, representation)
        
    def _assert_run_actions(self, expected_result, expected_run_list, mock_setup_list, force_execution):
        actions = []
        
        for mock_setup in mock_setup_list:
            satisfiable_requirement, can_run = mock_setup
            logger.info('Adding a new mock action with satisfiable_requirement=%s, can_run=%s', satisfiable_requirement, can_run)

            actions = actions + [MockAction(satisfiable_requirement, can_run)]

        usecase = MockStructureUsecase(actions)
        cache = MockCacheHolder([])
        
        logger.info('Expecting after execution: result=%s and run_actions=%s', expected_result, expected_run_list)
        
        result = usecase.run(cache, force_execution = force_execution)
        run_list = [action.has_run() for action in actions]
        logger.debug('Found after execution: result=%s and run_actions=%s', result, run_list)
        
        self.assertEquals(expected_result, result)
        self.assertEquals(expected_run_list, run_list)

class TestCompositeStructureUsecase(unittest.TestCase):
    def test_empty(self):
        usecase = usecases.CompositeStructureUsecase([])
        
        cache = MockCacheHolder([])
        
        self.assertEquals(0, len(usecase._get_actions()))
        
        self.assertTrue(usecase.check_all_requirements(cache))
        
        result = usecase.run(cache, True)
        self.assertTrue(result)
        
        result = usecase.run(cache, True)
        self.assertTrue(result)
        
    def test_actions(self):
        action_1 = MockAction(False, False)
        action_2 = MockAction(False, True)
        action_3 = MockAction(True, False)
        action_4 = MockAction(True, True)
        
        self._assert_actions([action_1, action_2, action_3, action_4], [[action_1, action_2, action_3, action_4]])

        self._assert_actions([action_1, action_2, action_3, action_4], [[], [action_1, action_2, action_3, action_4]])
        self._assert_actions([action_1, action_2, action_3, action_4], [[action_1], [action_2, action_3, action_4]])
        self._assert_actions([action_1, action_2, action_3, action_4], [[action_1, action_2], [action_3, action_4]])
        self._assert_actions([action_1, action_2, action_3, action_4], [[action_1, action_2, action_3], [action_4]])
        self._assert_actions([action_1, action_2, action_3, action_4], [[action_1, action_2, action_3, action_4], []])
        
        self._assert_actions([action_1, action_2, action_3, action_4], [[], [], [action_1, action_2, action_3, action_4]])
        self._assert_actions([action_1, action_2, action_3, action_4], [[action_1], [], [action_2, action_3, action_4]])
        self._assert_actions([action_1, action_2, action_3, action_4], [[action_1], [action_2], [action_3, action_4]])
        self._assert_actions([action_1, action_2, action_3, action_4], [[action_1], [action_2, action_3], [action_4]])
        self._assert_actions([action_1, action_2, action_3, action_4], [[action_1, action_2], [action_3], [action_4]])
        
    def _assert_actions(self, actions_expected, action_list_of_list):
        all_usecases = [MockStructureUsecase(actions) for actions in action_list_of_list]
        
        composite_usecase = usecases.CompositeStructureUsecase(all_usecases)
        
        self.assertEquals(actions_expected, composite_usecase._get_actions())
        
    def test_run_single_usecases(self):
        actions_1 = [MockAction(True, True)]*3
        actions_2 = [MockAction(True, True)]*3
        actions_3 = [MockAction(True, True)]*3
        
        composite_usecase = usecases.CompositeStructureUsecase(
            [MockStructureUsecase(actions_1), MockStructureUsecase(actions_2), MockStructureUsecase(actions_3)])
        
        self.assertEquals([False]*3, [action.has_run() for action in actions_1])
        self.assertEquals([False]*3, [action.has_run() for action in actions_2])
        self.assertEquals([False]*3, [action.has_run() for action in actions_3])
        
        cache = MockCacheHolder([])
        
        composite_usecase.run_single(0, cache)
        
        self.assertEquals([True]*3, [action.has_run() for action in actions_1])
        self.assertEquals([False]*3, [action.has_run() for action in actions_2])
        self.assertEquals([False]*3, [action.has_run() for action in actions_3])
        
        composite_usecase.run_single(1, cache)
        
        self.assertEquals([True]*3, [action.has_run() for action in actions_1])
        self.assertEquals([True]*3, [action.has_run() for action in actions_2])
        self.assertEquals([False]*3, [action.has_run() for action in actions_3])
        
        composite_usecase.run_single(2, cache)
        
        self.assertEquals([True]*3, [action.has_run() for action in actions_1])
        self.assertEquals([True]*3, [action.has_run() for action in actions_2])
        self.assertEquals([True]*3, [action.has_run() for action in actions_3])
        
    def test_run_all_usecases_000(self):
        self._assert_run_all_usecases(True, [[True]*3, [True]*3, [True]*3], 
                    [[MockAction(True, True)]*3, [MockAction(True, True)]*3, [MockAction(True, True)]*3])
        
    def test_run_all_usecases_001(self):
        self._assert_run_all_usecases(False, [[True]*3, [True]*3, [True, False, False]], 
                    [[MockAction(True, True)]*3, [MockAction(True, True)]*3, [MockAction(True, False), MockAction(True, False), MockAction(True, False)]])
        
    def test_run_all_usecases_002(self):
        self._assert_run_all_usecases(False, [[True]*3, [True]*3, [False, False, False]], 
                    [[MockAction(True, True)]*3, [MockAction(True, True)]*3, [MockAction(False, False), MockAction(False, False), MockAction(False, False)]])
        
    def test_run_all_usecases_01x(self):
        self._assert_run_all_usecases(False, [[True]*3, [True, False, False], [False, False, False]], 
                    [[MockAction(True, True)]*3, [MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(True, True)]*3])
        self._assert_run_all_usecases(False, [[True]*3, [True, False, False], [False, False, False]], 
                    [[MockAction(True, True)]*3, [MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(True, False)]*3])
        self._assert_run_all_usecases(False, [[True]*3, [True, False, False], [False, False, False]], 
                    [[MockAction(True, True)]*3, [MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(False, False)]*3])
        
        
    def test_run_all_usecases_02x(self):
        self._assert_run_all_usecases(False, [[True]*3, [False, False, False], [False, False, False]], 
                    [[MockAction(True, True)]*3, [MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(True, True)]*3])
        self._assert_run_all_usecases(False, [[True]*3, [False, False, False], [False, False, False]], 
                    [[MockAction(True, True)]*3, [MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(True, False)]*3])
        self._assert_run_all_usecases(False, [[True]*3, [False, False, False], [False, False, False]], 
                    [[MockAction(True, True)]*3, [MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(False, False)]*3])
        
    def test_run_all_usecases_1xx(self):
        self._assert_run_all_usecases(False, [[True, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(True, True)]*3, [MockAction(True, True)]*3])
        self._assert_run_all_usecases(False, [[True, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(True, True)]*3, [MockAction(True, False)]*3])
        self._assert_run_all_usecases(False, [[True, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(True, True)]*3, [MockAction(False, False)]*3])
        self._assert_run_all_usecases(False, [[True, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(True, False)]*3, [MockAction(True, True)]*3])
        self._assert_run_all_usecases(False, [[True, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(False, False)]*3, [MockAction(True, True)]*3])
        self._assert_run_all_usecases(False, [[True, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(True, False)]*3, [MockAction(True, False)]*3])
        self._assert_run_all_usecases(False, [[True, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(False, False)]*3, [MockAction(False, False)]*3])
        self._assert_run_all_usecases(False, [[True, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(True, False)]*3, [MockAction(False, False)]*3])
        self._assert_run_all_usecases(False, [[True, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(True, False), MockAction(True, False), MockAction(True, False)], [MockAction(False, False)]*3, [MockAction(True, False)]*3])
        
    def test_run_all_usecases_2xx(self):
        self._assert_run_all_usecases(False, [[False, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(True, True)]*3, [MockAction(True, True)]*3])
        self._assert_run_all_usecases(False, [[False, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(True, True)]*3, [MockAction(True, False)]*3])
        self._assert_run_all_usecases(False, [[False, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(True, True)]*3, [MockAction(False, False)]*3])
        self._assert_run_all_usecases(False, [[False, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(True, False)]*3, [MockAction(True, True)]*3])
        self._assert_run_all_usecases(False, [[False, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(False, False)]*3, [MockAction(True, True)]*3])
        self._assert_run_all_usecases(False, [[False, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(True, False)]*3, [MockAction(True, False)]*3])
        self._assert_run_all_usecases(False, [[False, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(False, False)]*3, [MockAction(False, False)]*3])
        self._assert_run_all_usecases(False, [[False, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(True, False)]*3, [MockAction(False, False)]*3])
        self._assert_run_all_usecases(False, [[False, False, False], [False, False, False], [False, False, False]], 
                    [[MockAction(False, False), MockAction(False, False), MockAction(False, False)], [MockAction(False, False)]*3, [MockAction(True, False)]*3])
        
    def _assert_run_all_usecases(self, expected_result, expected_run_list, action_list_of_list):
        all_usecases = [MockStructureUsecase(actions) for actions in action_list_of_list]
        
        composite_usecase = usecases.CompositeStructureUsecase(all_usecases)
        cache = MockCacheHolder([])
        
        self.assertEquals(expected_result, composite_usecase.run(cache))
        
        
        self.assertEquals(expected_run_list, [[action.has_run() for action in actions] for actions in action_list_of_list])
        
class MockStructureUsecase(usecases.StructureUsecase):
    def __init__(self, actions):
        usecases.StructureUsecase.__init__(self)

        self._action_list += actions

class MockRequirement(object):
    def __init__(self, satisfiable):
        self._satisfiable = satisfiable
        
    def __repr__(self):
        return self.__class__.__name__ + '(' + str(self._satisfiable) + ')'

    def __eq__(self, other):
        return type(other).__name__ == self.__class__.__name__ and self._satisfiable == other._satisfiable
        
    def check_requirement(self, cache):
        logger.debug('Mock requirement is satisfied=%s as requested', self._satisfiable)
        
        return self._satisfiable

class MockAction(usecases.StructureAction):
    def __init__(self, satisfiable_requirement, can_run):
        usecases.StructureAction.__init__(self)
   
        self._requirements += [MockRequirement(satisfiable_requirement)]
        self._satisfiable_requirement = satisfiable_requirement
        self._can_run = can_run
        self._has_run = False
        
    def has_run(self):
        return self._has_run
        
    def run(self, cache):
        logger.debug('Mock action can run=%s as requested', self._can_run)
        self._has_run = True

        return self._can_run
        
    def __repr__(self):
        return self.__class__.__name__ + '(' + str(self._satisfiable_requirement) + ', ' + str(self._can_run) + ')'
